from typing import Any, Callable
from datetime import datetime
import os
import re

import ipih

from pih import PIH, A, ActionValue, ActionStack, Input, Output, Session
from pih.consts import (
    FILE,
    PASSWORD,
    MarkType,
    SessionFlags,
    PasswordSettings,
    CheckableSections,
)
from pih.consts.errors import NotFound
from pih.collections import (
    Mark,
    User,
    Result,
    EventDS,
    UserBase,
    FullName,
    MarkGroup,
    ZabbixHost,
    Workstation,
    PrinterReport,
    FieldItemList,
    DiskStatistics,
    ResourceStatus,
    PersonDivision,
     ZabbixMetrics,
    EventDescription,
    RobocopyJobStatus,
    LoginPasswordPair,
    SiteResourceStatus,
    CTIndicationsValue,
    ZabbixMetricsValue,
    ComputerDescription,
    TimeSeriesStatistics,
    StorageVariableHolder,
    DisksStatisticsStatus,
    ExpiredTimestampVariableHolder,
)
from pih.tools import (
    j,
    e,
    i,
    lw,
    nl,
    js,
    nn,
    ne,
    nns,
    nnt,
    jnl,
    esc,
    one,
    b as mb,
    while_not_do,
    BitMask as BM,
)


LINE: str = "........................................................"


class ConsoleAppsApi:

    def __init__(self, pih: PIH | None = None):
        self.pih = pih or PIH
        self.full_name: FullName | None = None
        self.mark_tab_number: str | None = None
        self.telephone_number: str | None = None
        self.mark_person_division_id: int | None = None
        self.user_is_exists: bool = False
        self.login: str | None = None
        self.password: str | None = None
        self.corporate_email: str | None = None
        self.user_container: UserBase | None = None
        self.description: str | None = None
        self.use_template_user: bool = False
        self.need_to_create_mark: bool | None = None
        self.yes_no = self.input.yes_no

    def create_qr_code_for_mobile_helper_command(
        self,
        command: str | None = None,
        title: str | None = None,
        show_result: bool = True,
    ) -> str | None:
        command = command or self.input.input("Введите название команды")
        title = title or self.input.input("Введите заголовок")
        result: bool = A.A_QR.for_mobile_helper_command(
            command,
            title,
            os.path.join(
                A.PTH.MOBILE_HELPER.QR_CODE_FOLDER,
                A.PTH.replace_prohibited_symbols_from_path_with_symbol(
                    A.PTH.add_extension(command, FILE.EXTENSION.PNG)
                ),
            ),
            56,
        )
        qr_code_image_path: str = A.PTH_QR.mobile_helper_command(command)
        if show_result:
            if result:
                self.output.good(
                    nl(
                        jnl(
                            (
                                f"Файл qr кода {self.bold(title)} создан.",
                                "Путь к файлу:",
                                self.bold(qr_code_image_path),
                            ),
                        )
                    )
                )
            else:
                self.output.error("qr код не был создан")
        return qr_code_image_path

    def create_qr_code_for_card_registry_folder(
        self, card_registry_folder_name: str | None = None, show_result: bool = True
    ) -> list[str]:
        if e(card_registry_folder_name):
            card_registry_folder_name = self.input.input(
                f"Введите название папки (или несколько значений названий папок разделенной запятой или пробелом)"
            )
        card_registry_folder_name_list: list[str] = re.split(
            "\\W+", card_registry_folder_name
        )
        card_registry_folder_name_list = list(
            map(lambda item: str(item).strip(), card_registry_folder_name_list)
        )
        result_path_list: list[str] = []
        for card_registry_folder_name_item in card_registry_folder_name_list:
            card_registry_folder_name_item = A.D_F.polibase_person_card_registry_folder(
                card_registry_folder_name_item
            )
            result: bool = A.A_QR.for_polibase_person_card_registry_folder(
                card_registry_folder_name_item
            )
            qr_code_image_path: str = (
                PIH.PATH.QR_CODE.polibase_person_card_registry_folder(
                    card_registry_folder_name_item
                )
            )
            if show_result and len(card_registry_folder_name_list) == 1:
                if not result:
                    self.output.error("qr код не был создан")
            if result:
                result_path_list.append(qr_code_image_path)
        return result_path_list

    def disks_report(self, host: str | None = None) -> None:
        host = host or self.input.input("Введите название компьютера или сервера")
        with self.output.make_loading(text="Получение результата"):
            with self.output.make_indent(1, True):
                if A.C_R.accessibility_by_ping(host):
                    disk_statistics_list: list[DiskStatistics] = A.D.filter(
                        lambda item: item.size > 0,
                        A.EXC.get_disk_statistics_list(host),
                    )
                    if e(disk_statistics_list):
                        self.output.error("Хост не имеет дисков")
                    else:
                        self.output.write_line(
                            j(
                                (
                                    nl("Информация о дисках:"),
                                    nl(),
                                    A.CT_V.BLUE_ROMB,
                                    " ",
                                    nl("Свободное пространство"),
                                )
                            )
                        )
                        with self.output.make_indent(2, True):
                            for disk_statistics in disk_statistics_list:
                                self.output.write_line(
                                    j(
                                        (
                                            js(("", A.CT_V.BULLET, "")),
                                            self.bold(disk_statistics.name),
                                            ": ",
                                            round(
                                                100
                                                * disk_statistics.free_space
                                                / disk_statistics.size,
                                                2,
                                            ),
                                            js(
                                                (
                                                    "%",
                                                    A.CT_V.ARROW,
                                                    A.D_F.size(
                                                        disk_statistics.free_space
                                                    ),
                                                    "/",
                                                    A.D_F.size(disk_statistics.size),
                                                )
                                            ),
                                        )
                                    )
                                )
                else:
                    self.output.error("Хост не доступен или не найден")

    @property
    def is_mobile_cli(self) -> bool:
        return self.session.is_mobile and BM.has(self.session.flags, SessionFlags.CLI)

    def resources_and_indications_check(
        self,
        checkable_section_list: list[CheckableSections],
        ask_for_update_before: bool = False,
        force_update: bool = False,
        all: bool = False,
        additional_text: dict[CheckableSections, str | None] | None = None,
    ) -> None:
        def show_additional_text(section: CheckableSections) -> None:
            if nn(additional_text) and nn(additional_text[section]):
                self.output.write_line(additional_text[section])  # type: ignore

        section: CheckableSections
        with self.output.make_indent(1):
            if CheckableSections.RESOURCES in checkable_section_list or (
                CheckableSections.WS in checkable_section_list
                or CheckableSections.SERVERS in checkable_section_list
            ):

                def label_function(resource: ResourceStatus, _: int) -> str:
                    result: list[str] = []
                    accessable: bool = resource.accessable  # type: ignore
                    status: str = A.D_F.yes_no(accessable, True)
                    result.append(
                        js((status, self.bold(self.output.blue_str(resource.name))))  # type: ignore
                    )
                    if isinstance(resource, SiteResourceStatus):
                        if resource.check_free_space_status:
                            free_space_result_list: str = (
                                resource.free_space_status.split(" ")
                            )
                            result.append(
                                j(
                                    (
                                        " " * 6,
                                        "Cвободное место: ",
                                        free_space_result_list[0],
                                        "(",
                                        free_space_result_list[1],
                                        ")",
                                    )
                                )
                            )
                        if resource.check_certificate_status:
                            result.append(
                                j(
                                    (
                                        " " * 6,
                                        "Сертификат доступен до: ",
                                        resource.certificate_status,
                                    )
                                )
                            )
                    return jnl(result)

                force_update = force_update or (
                    ask_for_update_before and self.yes_no("Обновить перед получением")
                )
                if force_update:
                    self.output.write_line(
                        self.italic(
                            j(
                                (
                                    self.get_formatted_given_name(),
                                    "ожидайте получение результата...",
                                )
                            )
                        )
                    )
                for checkable_section in [
                    CheckableSections.RESOURCES,
                    CheckableSections.WS,
                    CheckableSections.SERVERS,
                ]:
                    if checkable_section in checkable_section_list:
                        if self.session.is_mobile:
                            self.output.new_line()
                        if all:
                            self.output.write_line(
                                js(
                                    (
                                        A.CT_V.BLUE_ROMB,
                                        self.bold(
                                            A.D.check(
                                                checkable_section
                                                == CheckableSections.RESOURCES,
                                                "Основные ресурсы",
                                                A.D.check(
                                                    checkable_section
                                                    == CheckableSections.WS,
                                                    "Наблюдаемые компьютеры",
                                                    "Сервера",
                                                ),
                                            )
                                        ),
                                    )
                                )
                            )
                        with self.output.make_indent(4, True):
                            self.output.write_result(
                                A.R_R.get_status_list(
                                    [checkable_section],
                                    force_update,
                                    (
                                        all
                                        if len(checkable_section_list) == 1
                                        and CheckableSections.WS
                                        in checkable_section_list
                                        else False
                                    ),
                                ),
                                False,
                                label_function=label_function,
                                separated_result_item=self.session.is_mobile,
                            )
            if CheckableSections.DISKS in checkable_section_list:
                if len(checkable_section_list) > 1:
                    if self.session.is_mobile:
                        self.output.new_line()
                    self.output.separated_line()
                    self.output.write_line(
                        js((A.CT_V.BLUE_ROMB, self.bold("Информация о дисках")))
                    )

                def label_function(resource: DisksStatisticsStatus, _: int) -> str:
                    result: list[str] = []
                    result.append(nl(self.bold(resource.host)))

                    def every_action(disk_statistics: DiskStatistics) -> None:
                        result.append(
                            j(
                                (
                                    js(("", A.CT_V.BULLET, "")),
                                    self.bold(disk_statistics.name),
                                    ": ",
                                    round(
                                        100
                                        * disk_statistics.free_space
                                        / disk_statistics.size,
                                        2,
                                    ),
                                    js(
                                        (
                                            "%",
                                            A.CT_V.ARROW,
                                            A.D_F.size(disk_statistics.free_space),
                                            "/",
                                            A.D_F.size(disk_statistics.size),
                                        )
                                    ),
                                )
                            )
                        )

                    A.D.every(
                        every_action,
                        resource.disk_list,
                    )

                    return jnl(result)

                with self.output.make_indent(2, True):
                    self.output.write_result(
                        A.R_R.get_status_list([CheckableSections.DISKS], force_update),
                        False,
                        label_function=label_function,
                        separated_result_item=self.session.is_mobile,
                    )
            if CheckableSections.INDICATIONS in checkable_section_list:
                with A.ER.detect():
                    if len(checkable_section_list) > 1:
                        if self.session.is_mobile:
                            self.output.new_line()
                        self.output.write_line(
                            js(
                                (
                                    A.CT_V.BLUE_ROMB,
                                    self.bold("Показания в помещении КТ"),
                                )
                            )
                        )
                    with self.output.make_indent(4):
                        self.output.write_result(A.R_IND.last_ct_value_containers(True))
                        self.output.write_image(
                            self.bold("График показаний"),
                            nns(A.D_CO.file_to_base64(
                                nns(A.PTH_STAT.get_file_path(A.CT_STAT.Types.CT_DAY))
                            )),
                        )
                    self.output.separated_line()
                    """
                    with self.output.make_indent(2):
                        self.output.write_line(
                            js(
                                (
                                    "",
                                    A.CT_V.BULLET,
                                    self.bold("Показания в техническом помещении МРТ"),
                                )
                            )
                        )
                    chiller_indications_value_container_result: Result[
                        list[ChillerIndicationsValueContainer]
                    ] = A.R_IND.last_chiller_value_containers(True)
                    chiller_indications_value_container: (
                        ChillerIndicationsValueContainer
                    ) = A.R.get_first_item(chiller_indications_value_container_result)
                    path: str = A.PTH_IND.CHILLER_DATA_IMAGE_LAST
                    modification_timstamp: float = A.PTH.get_modification_time(path)
                    file_creating_datetime: datetime | None = datetime.fromtimestamp(
                        modification_timstamp
                        if modification_timstamp > 0
                        else A.PTH.get_creation_time(path)
                    )
                    """
                    with self.output.make_indent(4):
                        """
                        if not A.C_IND.chiller_on():
                            self.output.write_line(
                                js(("", A.CT_V.WARNING, "Чиллер выключен"))
                            )
                        else:
                            with self.output.make_indent(2, True):
                                self.output.write_result(
                                    chiller_indications_value_container_result,
                                    title=(
                                        nl("Показания чиллера")
                                        if A.D_C.INDICATIONS.chiller_value_actual(
                                            chiller_indications_value_container
                                        )
                                        else nl(
                                            j(
                                                (
                                                    self.bold("Внимание!"),
                                                    ": ",
                                                    "Показания чиллера неактуальны",
                                                )
                                            )
                                        )
                                    ),
                                    separated_result_item=False,
                                )
                                self.output.write_image(
                                    js(
                                        (
                                            nl("Изображение дисплея чиллера"),
                                            "Время снятия:",
                                            A.D_F.datetime(file_creating_datetime),
                                        )
                                    ),
                                    A.D_CO.file_to_base64(
                                        A.PTH_IND.CHILLER_DATA_IMAGE_LAST_RESULT
                                    ),
                                )
                        """
                        self.output.write_line(
                            self.bold("Текущая выработка фильтра чиллера МРТ:")
                        )
                        statistics: TimeSeriesStatistics = A.D_STAT.for_chiller_filter()
                        total_seconds: int = (
                            A.D.now() - statistics.values[-1]
                        ).total_seconds()

                        def repr_value(title: str, value: int) -> str:
                            return js(
                                (
                                    "",
                                    A.CT_V.BULLET,
                                    j((title, A.CT.SPLITTER)),
                                    self.bold(
                                        js(
                                            (
                                                str(int(100 * (total_seconds / value))),
                                                "%",
                                            )
                                        )
                                    ),
                                )
                            )

                        self.output.write_line(
                            jnl(
                                (
                                    repr_value("Минимальная", statistics.max),
                                    repr_value("Средняя", statistics.avg),
                                    repr_value("Максимальная", statistics.min),
                                ),
                            )
                        )
            if CheckableSections.TIMESTAMPS in checkable_section_list:
                if len(checkable_section_list) > 1:
                    if self.session.is_mobile:
                        self.output.new_line()
                    self.output.separated_line()
                    self.output.write_line(
                        js((A.CT_V.BLUE_ROMB, self.bold("Временные метки")))
                    )
                with self.output.make_separated_lines():
                    with self.output.make_indent(2, True):
                        item: StorageVariableHolder | None = None
                        for item in A.D_V_T_E.get():
                            expired_timestamp_item: ExpiredTimestampVariableHolder = (
                                item.default_value
                            )
                            self.output.write_line(
                                js(
                                    (
                                        "",
                                        A.CT_V.BULLET,
                                        j((self.bold(item.description), A.CT.SPLITTER)),
                                        j(
                                            (
                                                nl(
                                                    nl(
                                                        js(
                                                            (
                                                                "  ",
                                                                A.CT_V.BULLET,
                                                                "Описание:",
                                                                A.D_F.whatsapp_send_message_to_it(
                                                                    js(
                                                                        (
                                                                            "note",
                                                                            esc(
                                                                                expired_timestamp_item.resolver_note
                                                                            ),
                                                                        )
                                                                    )
                                                                ),
                                                            )
                                                        )
                                                    ),
                                                    reversed=True,
                                                ),
                                                nl(
                                                    js(
                                                        (
                                                            "  ",
                                                            A.CT_V.BULLET,
                                                            "Дата установки:",
                                                            self.bold(
                                                                A.D.datetime_to_string(
                                                                    A.D_V_T_E.timestamp(
                                                                        expired_timestamp_item
                                                                    ),
                                                                    A.CT.DATE_FORMAT,
                                                                )
                                                            ),
                                                        )
                                                    )
                                                ),
                                                nl(
                                                    js(
                                                        (
                                                            "  ",
                                                            A.CT_V.BULLET,
                                                            "Дата окончания:",
                                                            self.bold(
                                                                A.D.datetime_to_string(
                                                                    A.D_V_T_E.value(
                                                                        expired_timestamp_item
                                                                    ),
                                                                    A.CT.DATE_FORMAT,
                                                                )
                                                            ),
                                                        )
                                                    )
                                                ),
                                                nl(
                                                    js(
                                                        (
                                                            "  ",
                                                            A.CT_V.BULLET,
                                                            "Действителен:",
                                                            self.bold(
                                                                A.D_V_T_E.left_life_time(
                                                                    expired_timestamp_item
                                                                )
                                                            ),
                                                            "дней",
                                                        )
                                                    )
                                                ),
                                            )
                                        ),
                                    )
                                )
                            )

            if CheckableSections.MATERIALIZED_RESOURCES in checkable_section_list:
                if len(checkable_section_list) > 1:
                    if self.session.is_mobile:
                        self.output.new_line()
                    self.output.separated_line()
                with self.output.make_indent(2, True):
                    for resource_type in A.CT_MR.Types:
                        if resource_type == A.CT_MR.Types.CHILLER_FILTER:
                            with self.output.make_indent(2, True):
                                self.output.write_line(
                                    nl(
                                        j(
                                            (
                                                self.bold("Количество"),
                                                ": ",
                                                str(A.D_MR.get_quantity(resource_type)),
                                                " штук",
                                            )
                                        )
                                    )
                                )
                                self.output.write_line(A.D_F.statistics(resource_type))
                                self.output.write_image(
                                    "График выработки по времени фильтров чиллера МРТ",
                                    A.D_CO.file_to_base64(
                                        A.PTH.STATISTICS.get_file_path(
                                            resource_type.name
                                        )
                                    ),
                                )
                                events_result: Result[list[EventDS]] = A.R_E.get_last(
                                    *A.E_B.action_was_done(
                                        A.CT_ACT.CHILLER_FILTER_CHANGING
                                    )
                                )
                                self.output.write_line(
                                    js(
                                        (
                                            "Прошло дней с последней замены фильтра:",
                                            self.bold(
                                                round(
                                                    (
                                                        A.D.now()
                                                        - one(events_result).timestamp
                                                    ).days
                                                )
                                            ),
                                        )
                                    )
                                )
            section = CheckableSections.POLIBASE
            if section in checkable_section_list:
                with self.output.make_indent(2, True):
                    self.output.write_image(
                        "Статистика дампа базы данных Полибейс",
                        A.D_CO.file_to_base64(
                            A.PTH_STAT.get_file_path(
                                A.CT_STAT.Types.POLIBASE_DATABASE_DUMP
                            )
                        ),
                    )
                    event_description: EventDescription = (
                        A.CT_E.POLIBASE_CREATION_DB_DUMP_COMPLETE
                    )
                    event: EventDS = one(A.R_E.get_last(event_description))
                    self.output.write_line(
                        js(
                            (
                                "",
                                A.CT_V.BULLET,
                                "Размер дампа бекапа базы данных:",
                                self.bold(
                                    A.D_F.size(
                                        event.parameters[
                                            A.D.get(event_description).params[0].name
                                        ],
                                        2,
                                        True,
                                    )
                                ),
                            )
                        )
                    )
                    event_description = (
                        A.CT_E.POLIBASE_CREATION_ARCHIVED_DB_DUMP_COMPLETE
                    )
                    event = one(A.R_E.get_last(event_description))
                    self.output.write_line(
                        js(
                            (
                                "",
                                A.CT_V.BULLET,
                                "Размер сжатого дампа бекапа базы данных:",
                                self.bold(
                                    A.D_F.size(
                                        event.parameters[
                                            A.D.get(event_description).params[0].name
                                        ],
                                        2,
                                        True,
                                    )
                                ),
                            )
                        )
                    )
                    show_additional_text(section)
                event_description: EventDescription = (
                    A.CT_E.POLIBASE_CREATION_DB_DUMP_COMPLETE
                )
                event: EventDS = one(A.R_E.get_last(event_description))
                hours: int = round(
                    (A.D.now() - event.timestamp).total_seconds() / 60 / 60
                )
                with self.output.make_separated_lines():
                    self.output.write_line(
                        j(
                            (
                                " ",
                                ([A.CT_V.GOOD, A.CT_V.ERROR][hours > 24]),
                                " ",
                                "Количество часов, прошедшее с создания дампа файла: ",
                                self.bold(hours),
                                ".",
                            )
                        )
                    )

            if CheckableSections.VALENTA in checkable_section_list:
                scanned_file_path_list: list[str] | None = (
                    A.PTH.file_path_list_by_directory_info(
                        A.PTH.MEDICAL_DATA.VALUE, confirmed=False
                    )
                )
                count: int = len(scanned_file_path_list)
                if self.session.is_mobile:
                    self.output.new_line()
                self.output.separated_line()
                self.output.write_line(
                    js(
                        (
                            A.CT_V.BLUE_ROMB,
                            self.bold(
                                j(
                                    (
                                        "Новые исследования в Валенте: ",
                                        (
                                            "Нет"
                                            if count == 0
                                            else j(("Да (", count, ")"))
                                        ),
                                    )
                                )
                            ),
                        )
                    )
                )
                if count > 0:
                    with self.output.make_indent(2, True):
                        if self.yes_no("Показать отсканированные изображения"):
                            for scanned_file_path in scanned_file_path_list:
                                self.output.write_image(
                                    j(
                                        (
                                            "Дата создания файла: ",
                                            A.D_F.datetime(
                                                datetime.fromtimestamp(
                                                    A.PTH.get_creation_time(
                                                        scanned_file_path
                                                    )
                                                )
                                            ),
                                        )
                                    ),
                                    A.D_CO.file_to_base64(scanned_file_path),
                                )
                        if self.yes_no("Синхронизировать Валенту"):
                            A.A_ACT.was_done(
                                A.CT_ACT.VALENTA_SYNCHRONIZATION, self.session.user
                            )
            if CheckableSections.PRINTERS in checkable_section_list:

                def create_printer_report(printer_report: PrinterReport) -> str:
                    report_list: list[str] = []
                    admin_description_list: list[str] = (
                        printer_report.adminDescription or ""
                    ).split(",")
                    name: str = A.D_F.host_name(printer_report.ip)
                    description_list: list[str] = A.D_F.format(
                        printer_report.description, use_python=True
                    ).split("\\n")
                    report_list.append(
                        j(
                            (
                                self.bold("Модель"),
                                ": ",
                                nl(printer_report.model),
                                self.bold("Название"),
                                ": ",
                                nl(name),
                                self.bold("Описание"),
                                ": ",
                                nl() if len(description_list) > 1 else "",
                                jnl(
                                    A.D.map(
                                        lambda item: (
                                            j((" ", A.CT_V.BULLET, " ", item))
                                            if len(description_list) > 1
                                            else item
                                        ),
                                        description_list,
                                    ),
                                ),
                            )
                        )
                    )

                    if (
                        printer_report.name not in (-401, -404)
                        and "infoless" not in admin_description_list
                    ):
                        report_list.append(self.bold("Статистика:"))
                        for item in (
                            ("Тонер", printer_report.get_toner),
                            (
                                None
                                if "drumless" in admin_description_list
                                else ("Драм-юнит", printer_report.get_drum)
                            ),
                        ):
                            if nn(item):
                                report_list.append(
                                    js(("", A.CT_V.BULLET, self.bold(item[0])))
                                )

                                def get_value(
                                    function: Callable[[str], int], color: str
                                ) -> str:
                                    value: int = function(color)
                                    result: str = js((str(value), "%"))
                                    if value < 5:
                                        result += j((" ", A.CT_V.WARNING))
                                    return result

                                report_list += [
                                    f"   {self.bold(color.upper())}: {get_value(item[1], color)}"
                                    for color in (
                                        ["k"]
                                        if "bw" in admin_description_list
                                        else ["c", "m", "y", "k"]
                                    )
                                ]
                    else:
                        report_list.append(
                            f"Принтер {A.D.check(printer_report.accessable, '', 'не ')}доступен"  # type: ignore
                        )
                    return jnl(report_list)

                with self.output.make_indent(1):
                    if len(checkable_section_list) > 1:
                        if self.session.is_mobile:
                            self.output.new_line()
                        self.output.separated_line()
                        self.output.write_line(
                            js((A.CT_V.BLUE_ROMB, self.bold("Отчет по принтерам")))
                        )

                    def action(balue: PrinterReport) -> None:
                        self.output.separated_line()
                        with self.output.make_indent(4, True):
                            self.output.write_line(create_printer_report(balue))

                    A.R.every(action, A.R_PR.report())

            if not all:
                if CheckableSections.BACKUPS in checkable_section_list:
                    robocopy_job_status_list: Result[list[RobocopyJobStatus]] = (
                        A.R_B.robocopy_job_status_list()
                    )
                    sort_by_status: bool = self.yes_no(
                        "Сортировать по статусу",
                        no_label=self.bold(
                            js(
                                (
                                    "Сортировать по дате выполнения",
                                    "-",
                                    A.CT_V.NUMBER_SYMBOLS[0],
                                )
                            )
                        ),
                    )
                    A.R.sort(
                        (
                            (
                                lambda item: item.last_status
                                or max(A.CT_RBK.STATUS_CODE.keys())
                            )
                            if sort_by_status
                            else lambda item: (
                                datetime.fromtimestamp(0)
                                if e(item.last_created)
                                else A.D.datetime_from_string(item.last_created)
                            )
                        ),
                        robocopy_job_status_list,
                        sort_by_status,
                    )

                    def job_status_item_label_function(
                        job_status: RobocopyJobStatus, index: int
                    ) -> str:
                        name: str = job_status.name
                        source: str = job_status.source
                        destination: str = job_status.destination
                        status: int | None = None
                        date: str | None = None
                        if job_status.active:
                            date = "выполняется"
                        else:
                            if job_status.last_created is not None:
                                date = f"{A.D_F.datetime(job_status.last_created)}"
                            status = job_status.last_status
                        variants: list[str] = [
                            "--" if status is None else self.bold(str(status)),
                            "--" if e(date) else self.bold(date),
                        ]
                        return j(
                            (
                                " ",
                                A.CT_V.BULLET,
                                " ",
                                variants[not sort_by_status],
                                ": ",
                                variants[sort_by_status],
                                j(
                                    (
                                        nl(),
                                        "   ",
                                        name,
                                        ": ",
                                        source,
                                        A.CT_V.ARROW,
                                        destination,
                                    )
                                ),
                            )
                        )

                    variants: list[str] = [
                        self.bold("Статус"),
                        self.bold("Дата выполнения"),
                    ]
                    self.output.write_result(
                        robocopy_job_status_list,
                        False,
                        label_function=job_status_item_label_function,
                        separated_result_item=False,
                        title=j(
                            (
                                " ",
                                A.CT_V.BULLET,
                                " ",
                                variants[not sort_by_status],
                                ": ",
                                variants[sort_by_status],
                                nl(),
                                "   Название Robocopy-задания",
                                nl(),
                                LINE,
                            )
                        ),
                    )

    def show_zabbix(self) -> None:
        def label_function(
            item: ZabbixHost | ZabbixMetrics | ZabbixMetricsValue | str,
            item2: ZabbixMetrics | int,
        ) -> str:
            if isinstance(item, ZabbixHost):
                return self.bold(lw(item.name))
            if isinstance(item, ZabbixMetrics):
                description: str | None = (
                    None if e(item.description) else j((" (", item.description, ")"))
                )
                return j(
                    (
                        item.itemid,
                        ": ",
                        item.name,
                        " [",
                        item.key_,
                        "] ",
                        description,
                        " ",
                        A.CT_V.ARROW,
                        " ",
                        item.lastvalue,
                        " ",
                        item.units,
                    )
                )
            if isinstance(item, ZabbixMetricsValue):
                return j((self.bold(A.D_F.datetime(item.clock)), ": ", item.value))

        host_id: int = int(
            self.input.item_by_index(
                "Выберите хост",
                A.D.sort(lambda item: lw(item.name), A.R_Z.hosts().data),
                label_function,
            ).id
        )
        zabbix_item_list: list[ZabbixMetrics] = A.R_Z.items(host_id).data
        zabbix_item_list_filtered: list[ZabbixMetrics] | None = None
        name: str | None = None
        if self.input.yes_no(
            j(("Найти метрику (общее количество: ", len(zabbix_item_list), ")")),
            yes_label="Введите значение для поиска метрики",
            no_label=js(
                (
                    "Показать весь список",
                    "-",
                    A.CT_V.NUMBER_SYMBOLS[0],
                )
            ),
        ):
            name = self.input.answer
            zabbix_item_list_filtered = A.D.filter(
                lambda item: A.D_C.zabbix_metrics_has_name(item, name),
                zabbix_item_list,
            )
            if e(zabbix_item_list_filtered):
                zabbix_item_list_filtered = zabbix_item_list
        metrics_list: list[ZabbixMetrics] = A.D.as_list(
            self.input.item_by_index(
                "Выберите метрику",
                zabbix_item_list_filtered or zabbix_item_list,
                label_function,
                allow_choose_all=ne(name),
            )
        )

        count: int = max(1, int(self.input.input("Введите количество значений")))

        def every_action(metrics: ZabbixMetrics) -> None:
            value_list_result: Result[list[ZabbixMetricsValue]] = A.R_Z.values(
                host_id,
                metrics.itemid,
                count,
            )
            with self.output.make_separated_lines():
                self.output.write_line(
                    j(
                        (
                            " ",
                            A.CT_V.BULLET,
                            " Метрика: ",
                            metrics.name,
                            " [",
                            metrics.key_,
                            "]",
                        )
                    )
                )
            with self.output.make_indent(2, True):
                A.R.every(
                    lambda item: self.output.write_line(
                        label_function(item, metrics_list)
                    ),
                    value_list_result,
                )

        A.D.every(every_action, metrics_list)

    def polibase_restart(self) -> None:
        if self.yes_no("Перезапустить сервер Polibase"):
            check_word: str = A.CT_P.NAME
            test: bool = not (
                check_word
                == self.input.input(
                    f"Введите контрольное слово: {self.bold(check_word)}"
                )
            )
            notify: bool = test or self.yes_no("Уведомить пользователей", True)
            title: str = f"Перезапуск Polibase{' test' if test else ''}"
            polibase_host: str = (
                A.CT_H.POLIBASE_TEST.NAME if test else A.CT_H.POLIBASE.NAME
            )
            self.output.write_line(
                A.L.it(
                    f"{title}: Начат процесс закрытия программы Polibase на компьютерах."
                )
            )
            A.A_P.client_program_close_for_all(notify, None, test)
            self.output.new_line()
            self.output.write_line(
                A.L.it(f"{title}: Начат процесс перезагрузки сервера Polibase.")
            )
            A.A_P.restart(test)
            while_not_do(
                lambda: not A.C_R.accessibility_by_ping(polibase_host), sleep_time=5
            )
            self.output.new_line()
            self.output.write_line(
                A.L.it(f"{title}: Начат процесс загрузки сервера Polibase.")
            )
            while_not_do(
                lambda: A.C_R.accessibility_by_ping(polibase_host), sleep_time=5
            )
            self.output.new_line()
            A.E.wait_server_start(polibase_host)
            self.output.write_line(
                A.L.it(f"{title}: Завершен процесс загрузки сервера Polibase.")
            )
            A.ME_P.notify_about_polibase_restarted(test)

    def polibase_client_program_close(
        self, search_value: str | None = None, for_all: bool = False
    ) -> None:
        if for_all:
            check_word: str = A.CT_P.NAME
            test: bool = not (
                check_word
                == (
                    search_value
                    or self.input.input(
                        f"Введите контрольное слово - {self.bold(check_word)}"
                    )
                )
            )
            A.A_P.client_program_close_for_all(
                True,
                self.input.input("Введите сообщение для пользователей Polibase"),
                test,
            )
        else:
            try:
                workstation_list: Result[list[Workstation]] = A.R_WS.by_any(
                    search_value
                    or self.input.input("Введите запрос для поиска компьютера")
                )

                def every_action(workstation: Workstation) -> None:
                    if A.A_P.client_program_close_for_workstation(workstation):
                        self.output.good(
                            f"Программа Polibase закрыта на компьютере {workstation.name}"
                        )

                A.R.every(every_action, workstation_list)
            except NotFound as error:
                self.output.error(error.get_details())
                search_value = None

    def process_kill(self, host_name: str | None, process_name: str | None) -> None:
        try:
            workstation_list: Result[list[Workstation]] = A.R_WS.by_any(
                host_name or self.input.input("Введите запрос для поиска компьютера")
            )

            def every_action(workstation: Workstation, process_name: str) -> None:
                process_value: int | str = process_name
                try:
                    process_value = int(process_value)
                except ValueError:
                    pass
                if A.A_WS.kill_process(process_value, workstation.name):
                    self.output.good(
                        j(
                            (
                                A.D.check(
                                    isinstance(process_value, str),
                                    j(("Процесс с именем ", '"', process_value, '" ')),
                                    js(
                                        (
                                            "Программа с идентификационным номером",
                                            process_value,
                                        )
                                    ),
                                ),
                                js(("закрыта на компьютере", workstation.name)),
                            )
                        )
                    )
                else:
                    self.output.error("Процесс не найден")

            A.R.every(
                lambda workstation: every_action(
                    workstation,
                    process_name
                    or self.input.input("Введите название процесса или его PID"),
                ),
                workstation_list,
            )
        except NotFound as error:
            self.output.error(error.get_details())
            host_name = None

    @property
    def output(self) -> Output:
        return self.pih.output

    @property
    def input(self) -> Input:
        return self.pih.input

    def send_whatsapp_message(self, telephone_number: str, message: str) -> bool:
        return A.ME_WH_W.send(telephone_number, message, A.CT_ME_WH_W.Profiles.IT)

    def mark_find(self, value: str | None = None) -> None:
        self.output.mark.by_any(value or self.input.mark.any())

    def arg(
        self,
        index: int = 0,
        default_value: Any | None = None,
    ) -> Any:
        return self.session.arg(index, default_value)

    def register_ct_indications(self) -> None:
        text: str = (
            f"число, которое может содержать дробную часть разделенную {self.bold('точкой')} или {self.bold('запятой')}"
        )
        number_format_notification_text: str = f"- {self.italic(text)}"

        def float_check_function(
            value: str | None, show_error: bool = True
        ) -> str | None:
            result: float | None = None
            if value is not None:
                result = A.D_Ex.float(value)
            if show_error and result is None:
                self.output.error(
                    f"Введите {self.bold('число')} {number_format_notification_text}"
                )
            return None if result is None else str(result)

        temperature: float = float_check_function(
            self.arg(), False
        ) or self.input.input(
            f"Введите значение {self.bold('температуры')} {number_format_notification_text}",
            check_function=float_check_function,  # type: ignore
        )  # type: ignore
        humidity: float = float_check_function(self.arg(1), False) or self.input.input(
            f"Введите значение {self.bold('влажности')} {number_format_notification_text}",
            check_function=float_check_function,  # type: ignore
        )  # type: ignore
        indications_value: CTIndicationsValue = CTIndicationsValue(
            temperature, humidity
        )
        if A.A_IND_CT.register(indications_value):
            with self.output.make_send_to_group(A.CT_ME_WH.GROUP.CT_INDICATIONS):
                self.output.write_result(
                    Result(A.CT_FC.INDICATIONS.CT_VALUE, indications_value),
                    title=f"{self.get_formatted_given_name()}, отправил следующие показания в помещение КТ:",
                )
            self.output.good("Спасибо, показания отправлены")

    def find_free_mark(self, value: str | None = None) -> None:
        self.output.mark.result(
            A.R_M.by_any(value or self.input.mark.any()),
            "Список свободных карт доступа:",
        )

    def find_user(self, value: str | None = None) -> None:
        try:
            self.output.user.result(
                A.R_U.by_any(value or self.input.user.title_any()),
                "Найденые пользователи:",
            )
        except NotFound as error:
            self.output.error(error.get_details())

    def create_password(self) -> str:
        password: str | None = None
        password_settings: PasswordSettings = PASSWORD.get(
            self.input.indexed_field_list(
                "Выберите тип пароля", A.CT_FC.POLICY.PASSWORD_TYPE
            )
        )
        while True:
            password = self.input.user.generate_password(True, password_settings)
            self.output.value("Пароль", password)
            if self.yes_no("Использовать", True):
                break
        if self.yes_no("Отправить в ИТ отдел"):
            A.L.it(f"Сгенерированный пароль:")
            A.L.it(password)
        return password

    def make_mark_as_free(self, value: str | None = None, confirm: bool = True) -> None:
        mark: Mark = self.input.mark.by_any(value)
        mark_type: int = A.D.get(MarkType, mark.type)
        if mark_type == MarkType.FREE:
            self.output.error("Карта доступа с введенным номером уже свободная")
        else:
            if not confirm or self.yes_no("Сделать карту свободной"):
                if mark_type == MarkType.TEMPORARY:
                    temporary_tab_number: int = mark.TabNumber
                    mark = A.R_M.temporary_mark_owner(mark).data
                if A.A_M.make_as_free_by_tab_number(mark.TabNumber):
                    if mark_type == MarkType.TEMPORARY:
                        A.E.it_notify_about_temporary_mark_return(
                            mark, temporary_tab_number
                        )
                    else:
                        A.E.it_notify_about_mark_return(mark)
                    self.output.good(
                        f"Карта доступа с номером {mark.TabNumber} стала свободной"
                    )
                else:
                    self.output.error("Ошибка")
            else:
                self.output.error("Отмена")

    def who_lost_the_mark(self, tab_number: str | None = None):
        try:
            tab_number = tab_number or self.input.tab_number()
            if tab_number is not None:
                try:
                    mark: Mark = A.R_M.by_tab_number(tab_number).data
                    mark_type: MarkType = A.D.get(MarkType, mark.type)
                    if mark_type == MarkType.FREE:
                        self.output.good("Это свободная карта доступа")
                    elif mark_type == MarkType.GUEST:
                        self.output.good("Это гостевая карта доступа")
                    else:
                        if mark_type == MarkType.TEMPORARY:
                            mark = A.R_M.temporary_mark_owner(mark).data
                            tab_number = mark.TabNumber
                            self.output.good("Это временная карта доступа")
                        if mark is not None:
                            telephone_number: str = mark.telephoneNumber
                            self.output.value("Персона", mark.FullName)
                            if not A.C.telephone_number(telephone_number):
                                user: User = A.R_U.by_tab_number(tab_number).data
                                if user is not None:
                                    telephone_number = user.telephoneNumber
                            if not A.C.telephone_number(telephone_number):
                                self.output.error(f"Телефон не указан")
                            else:
                                self.output.value("Телефон", telephone_number)
                                if self.yes_no("Отправить сообщение", True):
                                    details: str = self.input.input(
                                        f"{self.get_formatted_given_name()}, уточните, где забрать найденную карту"
                                    )
                                    if self.send_whatsapp_message(
                                        telephone_number,
                                        f"День добрый, {A.D.to_given_name(mark.FullName)}, вашу карту доступа ({tab_number}) нашли, заберите ее {details}",
                                    ):
                                        self.output.good("Сообщение отправлено")
                                    else:
                                        self.output.error(
                                            "Ошибка при отправке сообщения"
                                        )
                        else:
                            self.output.error("Телефон не указан")
                except NotFound:
                    self.output.error("Карта доступа, с введенным номером не найдена")
        except KeyboardInterrupt:
            pass

    def bold(self, value: str) -> str:
        return self.output.bold(value)

    def italic(self, value: str) -> str:
        return self.output.italic(value)

    def create_new_mark(self):
        self.full_name: str | None = None
        self.mark_tab_number: str | None = None
        self.telephone_number: str | None = None
        self.mark_person_division_id: int | None = None

        def get_full_name() -> ActionValue:
            self.output.header("Заполните ФИО персоны")
            self.full_name = self.input.full_name(True)
            user_exists: bool = not A.C.MARK.exists_by_full_name(self.full_name)
            if user_exists:
                self.output.error(
                    "Персона с данной фамилией, именем и отчеством уже есть!"
                )
                if not self.yes_no("Продолжить"):
                    self.session.exit()
            return self.output.get_action_value(
                "ФИО персоны", A.D.fullname_to_string(self.full_name)
            )

        def get_telephone_number() -> ActionValue:
            self.output.header("Заполните номер телефона")
            self.telephone_number = self.input.telephone_number()
            return self.output.get_action_value(
                "Номер телефона", self.telephone_number, False
            )

        def get_tab_number() -> ActionValue:
            self.output.header("Выбор группы и номера для карты доступа")
            free_mark: Mark = self.input.mark.free()
            group_name: str = free_mark.GroupName
            self.mark_tab_number = free_mark.TabNumber
            self.output.value("Группа карты доступа", group_name)
            return self.output.get_action_value(
                "Номер карты пропуска", self.mark_tab_number
            )

        def get_division() -> ActionValue:
            self.output.header("Выбор подразделения")
            person_division: PersonDivision = self.input.mark.person_division()
            self.mark_person_division_id = person_division.id
            return self.output.get_action_value(
                "Подразделение, к которому прикреплена персона", person_division.name
            )

        ActionStack(
            "Данные пользователя",
            get_full_name,
            get_division,
            get_telephone_number,
            get_tab_number,
            input=self.input,
            output=self.output,
        )
        if self.yes_no("Создать карту доступа для персоны", True):
            if A.A_M.create(
                self.full_name,
                self.mark_person_division_id,
                self.mark_tab_number,
                self.telephone_number,
            ):
                self.output.good("Карты доступа создана!")
                A.E.it_notify_about_create_new_mark(self.full_name)
                if self.yes_no("Уведомить персону", True):
                    self.send_whatsapp_message(
                        self.telephone_number,
                        f"Сообщение от ИТ отдела Pacific International Hospital: День добрый, {A.D.to_given_name(self.full_name)}, Вам выдана карта доступа с номером {self.mark_tab_number}",
                    )
            else:
                self.output.error("Карта доступа не создана!")

    def send_workstation_message_to_all(self) -> None:
        message: str = self.input.message(
            f"{self.get_formatted_given_name()}, введите сообщение для всех пользователей"
        )
        A.ME_WS.to_all_workstations(message, None, [A.CT_H.WS255], self.session)

    @property
    def user(self) -> User:
        return self.session.user

    @property
    def user_description(self) -> str:
        return A.D_F.description(self.user.description)

    def send_workstation_message(
        self,
        recipient_name: str | None = None,
        message: str | None = None,
        ask_for_use_dialog: bool = True,
    ) -> None:
        use_dialog: bool = False
        recipient: User | ComputerDescription | None = None
        while True:
            try:
                recipient = A.D.get_first_item(
                    self.input.user.by_any(recipient_name, True)
                )
                if nn(recipient):
                    break
            except NotFound as error:
                value: str = error.get_value()
                if A.C_WS.name(value):
                    if A.C_WS.exists(value):
                        recipient = A.R_WS.by_name(value).data
                        break
                if recipient is None:
                    recipient_name = None
                    self.output.error(error.get_details())
        if isinstance(recipient, User) and A.R.is_empty(
            A.R_WS.by_login(recipient.samAccountName)
        ):
            self.output.error(
                js(
                    (
                        "Пользователь ",
                        recipient.name,
                        " (",
                        recipient.samAccountName,
                        ") ",
                        "не залогинен ни за одним компьютером.",
                    )
                )
            )
        else:
            try:
                if ask_for_use_dialog:
                    use_dialog = e(message) and self.yes_no("Начать диалог")
                else:
                    use_dialog = True
                while True:
                    prefix: str | None = None
                    if isinstance(recipient, User):
                        prefix = j(
                            (
                                "Сообщение от ",
                                self.user_given_name,
                                " (",
                                self.user_description,
                                "): ",
                                A.D.to_given_name(recipient),
                                ", ",
                            )
                        )
                        message = message or self.input.message(
                            js(
                                (
                                    "введите сообщение для пользователя",
                                    self.get_formatted_given_name(
                                        A.D.to_given_name(recipient)
                                    ),
                                )
                            ),
                            None if self.session.is_mobile else prefix,
                        )
                        if A.ME_WS.to_user(recipient, message):
                            message = None
                            self.output.good("Сообщение отправлено")
                    else:
                        prefix = j(
                            (
                                "Сообщение от ",
                                self.user_given_name,
                                " (",
                                self.user_description,
                                "): ",
                            )
                        )
                        if self.session.is_mobile:
                            self.output.write_line(nl(i(prefix)))
                        message = message or self.input.message(
                            f"введите сообщение для компьютера {recipient.name}",
                            None if self.session.is_mobile else prefix,
                        )
                        if A.ME_WS.to_workstation(recipient, message):
                            self.output.good("Сообщение отправлено")
                            message = None
                    if use_dialog:
                        self.output.separated_line()
                    else:
                        break
            except KeyboardInterrupt as error:
                if use_dialog:
                    self.output.error("Выход из диалога...")
                else:
                    self.output.error("Отмена...")
                raise error

    @property
    def user_given_name(self) -> str:
        return self.session.user_given_name

    def get_formatted_given_name(self, value: str | None = None) -> str:
        return self.output.user.get_formatted_given_name(value or self.user_given_name)

    def create_temporary_mark(self, owner_mark: Mark | None = None) -> None:
        owner_mark = owner_mark or self.input.mark.by_any()
        mark_group: MarkGroup | None = None
        if self.yes_no("Выдать временную карту доступа из той же группы доступа"):
            mark_group = owner_mark
        temporary_mark: Mark = self.input.mark.free(mark_group)  # type: ignore
        self.output.temporary_candidate_for_mark(temporary_mark)
        full_name: str = owner_mark.FullName  # type: ignore
        tab_number: str = temporary_mark.TabNumber  # type: ignore
        if self.yes_no(
            f"Создать временную карту для {full_name} с табельным номеров {tab_number}",
            True,
        ):
            if A.A_M.make_as_temporary(temporary_mark, owner_mark):
                self.output.good("Временная карта создана")
                telephone_number: str = owner_mark.telephoneNumber  # type: ignore
                A.E.it_notify_about_create_temporary_mark(full_name, tab_number)
                if not A.C.telephone_number(telephone_number):
                    user: User = A.R.get_first_item(A.R_U.by_any(owner_mark))  # type: ignore
                    if user is not None:
                        telephone_number = user.telephoneNumber  # type: ignore
                if A.C.telephone_number(telephone_number):
                    if self.yes_no("Уведомить персону", True):
                        self.send_whatsapp_message(
                            telephone_number,
                            f"Сообщение от ИТ отдела: День добрый, {self.get_formatted_given_name(full_name)}, Вам выдана временная карта доступа с номером {tab_number}",
                        )
            else:
                self.output.error("Ошибка при создании временной карты")
        else:
            self.output.error("Отмена")

    def telephone_number_fix_action(self, user: User) -> None:
        try:
            telephone: str = self.input.telephone_number()
            if A.A_U.set_telephone_number(user, telephone):
                self.output.good("Сохранен")
                self.output.line()
            else:
                self.output.error("Ошибка")
        except KeyboardInterrupt:
            self.output.new_line()
            self.output.error("Отмена")
            self.output.new_line()

    def start_user_telephone_number_editor(self) -> None:
        only_empty_telephone_number_edit: bool = self.yes_no(
            "Редактировать только телефоны, которые не заданы", True
        )
        result: Result[list[User]] = A.R_U.all(True)
        for user in result.data:  # type: ignore
            user: User = user
            if A.C_U.user(user):
                if user.telephoneNumber is None:
                    self.output.error(f"{user.name}: нет телефона")
                    self.telephone_number_fix_action(user)
                elif not A.C.telephone_number(user.telephoneNumber):
                    fixed_telephone: str = A.D_F.telephone_number(user.telephoneNumber)  # type: ignore
                    if A.C.telephone_number(fixed_telephone):
                        self.output.good(f"{user.name} телефон исправлен")
                        A.A_U.set_telephone_number(user, fixed_telephone)
                    else:
                        self.output.yellow(
                            f"{user.name}: неправильный формат телефона ({user.telephoneNumber})"
                        )
                else:
                    if not only_empty_telephone_number_edit:
                        self.output.good(f"{user.name}: телефон присутствует")
                        self.telephone_number_fix_action(user)
            else:
                self.output.notify(
                    f"{user.name}, похоже не пользователь, у которого должен быть номер телефона"
                )

    def start_user_property_setter(
        self,
        property_name: str,
        search_value: str | None = None,
        choose_user: bool = False,
    ) -> None:
        try:
            user_list: list[User] | None = None
            fields: FieldItemList = A.CT_FC.AD.USER
            active: bool | None = (
                True
                if (
                    property_name == A.CT_UP.PASSWORD
                    or property_name == A.CT_UP.TELEPHONE_NUMBER
                )
                else None
            )
            if choose_user:
                user_list = self.input.user.by_any(search_value, active)
            else:
                result: Result[list[User]] = A.R_U.by_any(
                    self.input.user.title_any(), active
                )
                user_list = result.data
            if property_name == A.CT_UP.USER_STATUS:
                for status in [
                    A.CT_AD.ACTIVE_USERS_CONTAINER_DN,
                    A.CT_AD.INACTIVE_USERS_CONTAINER_DN,
                ]:
                    work_user_list: list[User] = A.D.FILTER.users_by_dn(
                        user_list,  # type: ignore
                        (
                            A.CT_AD.INACTIVE_USERS_CONTAINER_DN
                            if status == A.CT_AD.ACTIVE_USERS_CONTAINER_DN
                            else A.CT_AD.ACTIVE_USERS_CONTAINER_DN
                        ),
                    )
                    for index, user in enumerate(work_user_list):
                        try:
                            self.output.user.result(Result(fields, [user]))
                            if self.yes_no(
                                js(
                                    (
                                        ["Деактивировать", "Активировать"][
                                            status == A.CT_AD.ACTIVE_USERS_CONTAINER_DN
                                        ],
                                        "пользователя",
                                    )
                                )
                            ):
                                if status == A.CT_AD.ACTIVE_USERS_CONTAINER_DN:
                                    if self.yes_no(
                                        "Использовать шаблон для пользователя", True
                                    ):
                                        user_container = self.input.user.template()
                                    else:
                                        user_container = self.input.user.container()
                                else:
                                    user_container = UserBase(
                                        distinguishedName=A.CT_AD.INACTIVE_USERS_CONTAINER_DN
                                    )
                                if A.A_U.set_status(user, status, user_container):
                                    self.output.good("Успешно")
                                else:
                                    self.output.error("Ошибка")
                            else:
                                self.output.new_line()
                                self.output.error("Отмена")
                        except KeyboardInterrupt:
                            self.output.new_line()
                            if index == len(user_list) - 1:  # type: ignore
                                self.output.error("Отмена")
                            else:
                                self.output.error("Отмена - следующий")
                            self.output.new_line()
            else:
                for index, user in enumerate(user_list):  # type: ignore
                    try:
                        if property_name == A.CT_UP.TELEPHONE_NUMBER:
                            self.output.user.result(
                                Result(fields, [user]), "Найденный пользователь"
                            )
                            telephone = self.input.telephone_number()
                            if A.C.telephone_number(telephone) and self.yes_no(
                                "Установить", True
                            ):
                                if A.A_U.set_telephone_number(user, telephone):
                                    self.output.good("Телефон установлен")
                                else:
                                    self.output.error("Ошибка установки телефона")
                            else:
                                self.output.error("Отмена")
                        elif property_name == A.CT_UP.PASSWORD:
                            self.output.user.result(
                                Result(fields, [user]), "Пользователи:"
                            )
                            password: str | None = None
                            while True:
                                password = self.input.user.generate_password(
                                    True,
                                    PASSWORD.get(
                                        self.input.indexed_field_list(
                                            "Выберите тип пароля",
                                            A.CT_FC.POLICY.PASSWORD_TYPE,
                                        )
                                    ),
                                )
                                self.output.value("Пароль", password)
                                if self.yes_no("Использовать", True):
                                    break
                            if self.yes_no("Установить", True):
                                if A.A_U.set_password(user, password):
                                    self.output.good("Успешно")
                                else:
                                    self.output.error("Ошибка")
                            else:
                                self.output.error("Отмена")
                    except KeyboardInterrupt:
                        self.output.new_line()
                        self.output.error(
                            "Отмена"
                            + (" - следующий" if index != len(user_list) - 1 else "")  # type: ignore
                        )
                        self.output.new_line()
        except NotFound as error:
            self.output.error(error.get_details())

    @property
    def session(self) -> Session:
        return self.pih.session

    def b(self, value: Any) -> str:
        return mb(value) if self.session.is_mobile else str(value)

    def create_new_user(self) -> None:
        self.full_name: FullName | None = None
        self.mark_tab_number: str | None = None
        self.telephone_number: str | None = None
        self.mark_person_division_id: int | None = None
        self.user_is_exists: bool = False
        self.login: str | None = None
        self.password: str | None = None
        self.corporate_email: str | None = None
        self.user_container: User | None = None
        self.description: str | None = None
        self.additional_information: str | None = None
        self.use_template_user: bool | None = None
        self.need_to_create_mark: bool | None = None

        def get_full_name(as_pc_user: bool = True) -> ActionValue:
            self.output.head(
                js(("Заполнение ФИО", "пользователя" if as_pc_user else "персоны"))
            )
            with self.output.make_indent(2):
                self.full_name = self.input.full_name(True)
                self.user_is_exists = A.C.USER.exists_by_full_name(self.full_name)
                if self.user_is_exists:
                    self.output.error(
                        js(
                            (
                                "Пользователь" if as_pc_user else "Персона",
                                "с данной фамилией, именем и отчеством уже есть!",
                            )
                        )
                    )
                    if not self.yes_no("Продолжить"):
                        self.session.exit()
                return self.output.get_action_value(
                    js(("ФИО", "пользователя" if as_pc_user else "персоны")),
                    A.D.fullname_to_string(self.full_name),
                )

        def get_login(as_pc_user: bool = True) -> ActionValue:
            self.output.header(
                js(
                    (
                        "Создание логина для аккаунта",
                        "пользователя" if as_pc_user else "корпоративной почты",
                    )
                )
            )
            with self.output.make_indent(2):
                self.login = self.input.user.generate_login(self.full_name)  # type: ignore
                return self.output.get_action_value(
                    js(
                        (
                            "Логин для",
                            (
                                "аккаунта пользователя"
                                if as_pc_user
                                else "аккаунта корпоративной почты"
                            ),
                        )
                    ),
                    self.login,
                )

        def get_telephone_number() -> ActionValue:
            self.output.header("Заполнение номера телефона")
            with self.output.make_indent(2):
                self.telephone_number = self.input.telephone_number()
                return self.output.get_action_value(
                    "Номер телефона", self.telephone_number, False
                )

        def get_description(as_pc_user: bool = True) -> ActionValue:
            self.output.header(
                js(("Заполнение описания", "пользователя" if as_pc_user else "персоны"))
            )
            with self.output.make_indent(2):
                self.description = self.input.description()
                return self.output.get_action_value(
                    js(("Описание", "пользователя" if as_pc_user else "персоны")),
                    self.description,
                    False,
                )

        def get_additional_information() -> ActionValue:
            self.output.header("Заполнение дополнительную информацию")
            with self.output.make_indent(2):
                self.additional_information = self.input.input(
                    "Дополнительную информация"
                )
                return self.output.get_action_value(
                    "Дополнительную информация", self.additional_information, False
                )

        def get_template_user_container_or_user_container() -> ActionValue:
            self.output.header("Выбор контейнера для пользователя")
            with self.output.make_indent(2):
                self.user_container, self.use_template_user = (
                    self.input.user.template(),
                    True,
                )
                return self.output.get_action_value(
                    "Контейнер пользователя", self.user_container.description  # type: ignore
                )

        def get_password(as_pc_user: bool = True) -> ActionValue:
            self.output.header(
                js(
                    (
                        "Создание пароля для",
                        (
                            "аккаунта пользователя"
                            if as_pc_user
                            else "аккаунта корпоративной почты"
                        ),
                    )
                )
            )
            with self.output.make_indent(2):
                self.password = self.input.user.generate_password(
                    settings=PASSWORD.SETTINGS.PC
                )
                return self.output.get_action_value("Пароль", self.password, False)

        def get_corporate_email(as_pc_user: bool = True) -> ActionValue:
            with self.output.make_indent(2):
                self.corporate_email = A.D_F.email(self.login, use_default_domain=True)  # type: ignore
                return self.output.get_action_value(
                    js(
                        (
                            "Адресс корпоративной электронной почты для",
                            "пользователя" if as_pc_user else "персоны",
                        )
                    ),
                    self.corporate_email,
                )

        def get_mark_person_division() -> ActionValue | None:
            if self.need_to_create_mark and A.D.is_none(self.mark_person_division_id):
                self.output.header("Выбор подразделения")
                with self.output.make_indent(2):
                    mark_person_division: PersonDivision = (
                        self.input.mark.person_division(False)
                    )
                    self.mark_person_division_id = mark_person_division.id
                    return self.output.get_action_value(
                        "Подразделение персоны, которой принадлежит карта доступа",
                        mark_person_division.name,  # type: ignore
                    )
            return None

        def get_tab_number() -> ActionValue | None:
            full_name_string: str = A.D.fullname_to_string(self.full_name)  # type: ignore
            mark: Mark | None = A.R_M.by_name(full_name_string, True).data  # type: ignore
            with self.output.make_indent(2):
                self.output.header("Создание карты доступа")
                if nn(mark):
                    if self.yes_no(
                        f"Найдена карта доступа для персоны {full_name_string} с номером {mark.TabNumber}. Использовать",
                        True,
                    ):
                        self.need_to_create_mark = False
                        self.mark_tab_number = mark.TabNumber  # type: ignore
                        self.mark_person_division_id = mark.DivisionID  # type: ignore
                        return None
                self.need_to_create_mark = self.yes_no(
                    js(("Создать карту доступа для персоны", self.b(full_name_string))),
                    True,
                )
                if self.need_to_create_mark:
                    free_mark: Mark = self.input.mark.free()
                    self.mark_tab_number = free_mark.TabNumber
                    self.mark_person_division_id = free_mark.DivisionID
                    self.output.value("Группа карты доступа", free_mark.GroupName)  # type: ignore
                    if nn(free_mark.DivisionID):
                        self.output.value(
                            "Подразделение персоны, которой принадлежит карта доступа",
                            free_mark.DivisionName,  # type: ignore
                        )
                    return self.output.get_action_value(
                        "Номер карты доступа", self.mark_tab_number  # type: ignore
                    )
            return None

        PC_USER_INDEX: int = 0
        action_index: int = self.input.index(
            "Кого создать",
            [
                f"Пользователя компьютера, имеющего:\n    {A.CT_V.BULLET} полибейс\n    {A.CT_V.BULLET} почту\n    {A.CT_V.BULLET} карту доступа",
                f"Персоны, имеющей:\n    {A.CT_V.BULLET} почту\n    {A.CT_V.BULLET} карту доступа",
            ],
            label_function=lambda item, _: item,
        )  # type: ignore
        is_user: bool = action_index == PC_USER_INDEX
        if is_user:
            ActionStack(
                "Данные пользователя",
                get_full_name,
                get_login,
                get_password,
                get_telephone_number,
                get_description,
                get_additional_information,
                get_template_user_container_or_user_container,
                get_corporate_email,
                get_tab_number,
                get_mark_person_division,
                input=self.input,
                output=self.output,
            )
        else:
            ActionStack(
                "Данные персоны",
                lambda: get_full_name(False),
                lambda: get_login(False),
                lambda: get_password(False),
                get_telephone_number,
                lambda: get_description(False),
                lambda: get_corporate_email(False),
                get_tab_number,
                get_mark_person_division,
                input=self.input,
                output=self.output,
            )

        full_name_parts: list[str] = A.D.map(
            lambda item: item.lower(), self.full_name.as_list()  # type: ignore
        )
        test: bool = "test" in full_name_parts or "тест" in full_name_parts
        if test or not is_user or self.yes_no("Создать аккаунт для пользователя", True):
            if not test:
                if is_user:
                    A.A_U.create_from_template(
                        self.user_container.distinguishedName,  # type: ignore
                        self.full_name,  # type: ignore
                        self.login,  # type: ignore
                        self.password,  # type: ignore
                        self.description,  # type: ignore
                        self.telephone_number,  # type: ignore
                        self.corporate_email,  # type: ignore
                    )
                if self.need_to_create_mark:
                    self.mark_tab_number = (
                        self.mark_tab_number
                        or A.R_M.by_name(
                            A.D.fullname_to_string(self.full_name), True  # type: ignore
                        ).data.TabNumber  # type: ignore
                    )
                    A.A_M.create(
                        self.full_name,  # type: ignore
                        self.mark_person_division_id,  # type: ignore
                        self.mark_tab_number,
                        self.telephone_number,
                    )
                if A.A_DOC.create_for_user(
                    A.PTH_U.get_document_name(
                        A.D.fullname_to_string(self.full_name),  # type: ignore
                        self.login if self.user_is_exists else None,
                    ),
                    self.full_name,  # type: ignore
                    self.mark_tab_number,  # type: ignore
                    LoginPasswordPair(self.login, self.password) if is_user else None,
                    LoginPasswordPair(self.login, self.password) if is_user else None,
                    LoginPasswordPair(self.corporate_email, self.password),
                ):
                    if is_user:
                        A.E.it_notify_about_create_user(
                            self.login, self.password, self.additional_information  # type: ignore
                        )
                    else:
                        A.E.it_notify_about_create_person(
                            self.full_name,  # type: ignore
                            self.corporate_email,  # type: ignore
                            self.password,  # type: ignore
                            self.description,  # type: ignore
                            self.telephone_number,  # type: ignore
                        )
                    self.send_whatsapp_message(
                        self.telephone_number,  # type: ignore
                        j(
                            (
                                "День добрый, ",
                                self.bold(A.D.to_given_name(self.full_name)),  # type: ignore
                                nl("."),
                                "Мы - отдел информационных технологий ",
                                self.bold("Pacific International Hospital."),
                            )
                        ),
                    )
                    if self.need_to_create_mark:
                        A.E.it_notify_about_create_mark(self.full_name)  # type: ignore
                        self.send_whatsapp_message(
                            self.telephone_number,  # type: ignore
                            js(
                                (
                                    "Вас ожидает карта доступа с номером:",
                                    self.bold(self.mark_tab_number),  # type: ignore
                                    "в",
                                    self.bold("отделе кадров"),
                                )
                            ),
                        )
                    if is_user:
                        self.send_whatsapp_message(
                            self.telephone_number,  # type: ignore
                            js(
                                (
                                    nl("Данные Вашего аккаунта:"),
                                    A.CT_V.BULLET,
                                    "Логин:",
                                    nl(self.login),  # type: ignore
                                    A.CT_V.BULLET,
                                    "Пароль для всего:",
                                    nl(self.password),  # type: ignore
                                    A.CT_V.BULLET,
                                    "Электронная почта:",
                                    self.corporate_email,
                                )
                            ),
                        )
                    else:
                        self.send_whatsapp_message(
                            self.telephone_number,  # type: ignore
                            js(
                                (
                                    nl("Ваши данные:"),
                                    A.CT_V.BULLET,
                                    "Электронная почта:",
                                    nl(self.corporate_email),  # type: ignore
                                    A.CT_V.BULLET,
                                    "Пароль:",
                                    nl(self.password),  # type: ignore
                                )
                            ),
                        )
            A.A_EM.send(
                (
                    A.CT.TEST.EMAIL_ADDRESS
                    if test
                    else A.R.get_first_item(
                        A.R_U.by_job_position(A.CT_AD.JobPositions.HR)  # type: ignore
                    ).mail  # type: ignore
                ),  # type: ignore
                js(("Почта сотрудника", A.D.fullname_to_string(self.full_name))),  # type: ignore
                self.corporate_email,  # type: ignore
            )



import os
from enum import Enum


from pih.consts.ad import AD
from pih.consts.facade import *
from pih.consts.file import FILE
from ipih import ROOT_MODULE_NAME
from pih.consts.hosts import Hosts
from pih.consts.python import PYTHON
from pih.tools import j, n, PathTool, nn
from pih.consts.document import DocumentTypes


from setuptools.dist import Distribution

PATH_SPLITTER: str = "/"
PATH_DOUBLE_SPLITTER: str = "//"


class PATH_SHARE:
    NAME: str = "shares"
    PATH: str = os.path.join(AD.PATH_ROOT, NAME)


class PATH_FACADE:
    LINUX_MOUNT_POINT: str = j((PATH_SPLITTER, "mnt"))
    SHARED_POINT_PATH: str = os.path.join("S$", FACADE.NAME)
    LINUX_MOUNT_POINT_PATH: str = j((LINUX_MOUNT_POINT, FACADE.NAME), PATH_SPLITTER)
    FILES_NAME: str = ".files"
    TOOLS_NAME: str = ".tools"

    VALUE: str = j(
        (
            PATH_DOUBLE_SPLITTER,
            AD.DOMAIN_MAIN,
            PATH_SPLITTER,
            FACADE.NAME,
            PATH_SPLITTER,
        )
    )

    @staticmethod
    def SERVICE(service_name: str) -> str:
        return os.path.join(
            PATH_FACADE.VALUE,
            FACADE.SERVICE_NAME(service_name),
        )

    @staticmethod
    def SERVICE_FILES(standalone_name: str) -> str:
        return os.path.join(PATH_FACADE.FILES(), standalone_name)

    @staticmethod
    def SERVICE_TOOLS(standalone_name: str) -> str:
        return os.path.join(PATH_FACADE.TOOLS(), standalone_name)

    @staticmethod
    def FILES() -> str:
        return os.path.join(PATH_FACADE.VALUE, PATH_FACADE.FILES_NAME)

    @staticmethod
    def TOOLS() -> str:
        return os.path.join(PATH_FACADE.VALUE, PATH_FACADE.TOOLS_NAME)

    @staticmethod
    def STORAGE() -> str:
        return os.path.join(r"\\", Hosts.DC1.NAME, PATH_FACADE.SHARED_POINT_PATH)

    class DITRIBUTIVE:

        SPLITTER: str = "-"
        FOLDER_NAME: str = ".distr"
        PACKAGE_FOLDER_NAME: str = "all"
        DEFAULT_PACKAGE_TAG: str = "py3-none-any"

        @staticmethod
        def NAME(value: str, version: str | None = None) -> str:
            return j(
                (ROOT_MODULE_NAME, value, None if n(version) else j(("==", version))),
                PATH_FACADE.DITRIBUTIVE.SPLITTER,
            )

        @staticmethod
        def VALUE() -> str:
            return os.path.join(PATH_FACADE.VALUE, PATH_FACADE.DITRIBUTIVE.FOLDER_NAME)

        @staticmethod
        def PACKAGE_FOLDER(name: str, version: str | None = None) -> str:
            path_list: list[str] = [PATH_FACADE.DITRIBUTIVE.VALUE(), name]
            if nn(version):
                path_list.insert(1, PATH_FACADE.DITRIBUTIVE.PACKAGE_FOLDER_NAME)
            return os.path.join(*path_list)

        @staticmethod
        def PACKAGE(name: str, version: str | None) -> str:
            def wheel_name(name: str, version: str | None) -> str:
                tag: str = ""
                dist_name: str = ""
                try:
                    dist: Distribution = Distribution(
                        attrs={"name": name, "version": version}
                    )
                    bdist_wheel_cmd = dist.get_command_obj("bdist_wheel")
                    bdist_wheel_cmd.ensure_finalized()
                    dist_name = bdist_wheel_cmd.wheel_dist_name
                    tag = PATH_FACADE.DITRIBUTIVE.SPLITTER.join(
                        bdist_wheel_cmd.get_tag()
                    )
                except BaseException as _:
                    dist_name = PATH_FACADE.DITRIBUTIVE.SPLITTER.join((name, version))
                    tag = PATH_FACADE.DITRIBUTIVE.DEFAULT_PACKAGE_TAG
                return PathTool.add_extension(
                    j((dist_name, PATH_FACADE.DITRIBUTIVE.SPLITTER, tag)),
                    PYTHON.PACKAGE_EXTENSION,
                )

            return os.path.join(
                *[
                    PATH_FACADE.DITRIBUTIVE.PACKAGE_FOLDER(name, version),
                    wheel_name(
                        j((ROOT_MODULE_NAME, name), "_"),
                        version,
                    ),
                ]
            )

    class VIRTUAL_ENVIRONMENT:

        NAME_PREFIX: str = ".venv"


class PATH_DATA_STORAGE:
    NAME: str = ".data"
    VALUE: str = os.path.join(PATH_FACADE.VALUE, NAME)


class PATH_BUILD:
    NAME: str = ".build"
    VALUE: str = os.path.join(PATH_FACADE.VALUE, NAME)


class PATH_SCAN:
    NAME: str = "scan"
    VALUE: str = os.path.join(AD.PATH_ROOT, NAME)


class PATH_MEDICAL_DATA:

    HOST: str = Hosts.DC1.NAME
    NAME: str = "medicalData"
    VALUE: str = PathTool.path(os.path.join(AD.PATH_ROOT, NAME))


class PATH_SCAN_TEST:
    NAME: str = "test"
    VALUE: str = os.path.join(PATH_SCAN.VALUE, NAME)


class PATH_SCAN_SOURCE:
    NAME: str = "Исходники"
    VALUE: str = os.path.join(PATH_SCAN.VALUE, NAME)


class PATH_SCAN_RESULT:
    NAME: str = "Результат"
    VALUE: str = os.path.join(PATH_SCAN.VALUE, NAME)

    @staticmethod
    def get_path(type: DocumentTypes) -> str | None:
        if type == DocumentTypes.MEDICAL_DIRECTION:
            return os.path.join(PATH_SCAN_RESULT.VALUE, "Направления")


class PATH_WS_816_SCAN:

    HOST: str = Hosts.WS816.NAME
    NAME: str = "Scans"
    VALUE: str = j((PATH_DOUBLE_SPLITTER, os.path.join(HOST, NAME)))


class PATH_OMS:
    NAME: str = "oms"
    VALUE: str = os.path.join(AD.PATH_ROOT, NAME)


class PATH_MARKS:
    NAME: str = "MarkServise"
    VALUE: str = os.path.join(AD.PATH_ROOT, NAME)


class PATH_IT:
    NAME: str = "5. IT"
    NEW_EMPLOYEES_NAME: str = "New employees"
    ROOT: str = os.path.join(PATH_SHARE.PATH, NAME)

    @staticmethod
    def get_new_employee_path(name: str) -> str:
        return os.path.join(
            os.path.join(PATH_IT.ROOT, PATH_IT.NEW_EMPLOYEES_NAME), name
        )


class PATH_APP:
    NAME: str = "apps"
    FOLDER: str = os.path.join(PATH_FACADE.VALUE, NAME)


class PATH_DOCS:
    NAME: str = f"Docs{FACADE.SERVICE_FOLDER_SUFFIX}"
    FOLDER: str = os.path.join(PATH_FACADE.VALUE, NAME)


class PATH_FONTS:
    NAME: str = "fonts"
    FOLDER: str = os.path.join(PATH_DOCS.FOLDER, NAME)

    @staticmethod
    def get(name: str) -> str:
        from pih.tools import PathTool

        return os.path.join(
            PATH_FONTS.FOLDER,
            PathTool.add_extension(name, FILE.EXTENSION.TRUE_TYPE_FONT),
        )


class PATH_APP_DATA:
    NAME: str = "data"
    FOLDER: str = os.path.join(PATH_APP.FOLDER, NAME)

    OCR_RESULT_NAME: str = "ocr result"
    OCR_RESULT_FOLDER: str = os.path.join(FOLDER, OCR_RESULT_NAME)


class PATH_STATISTICS:
    NAME: str = "statistics"
    CHART_FILE_NAME_PREFIX: str = "chart_"
    FOLDER: str = os.path.join(PATH_APP_DATA.FOLDER, NAME)

    @staticmethod
    def get_file_path(name: str | Enum) -> str:
        from pih.tools import PathTool, EnumTool, j

        name = EnumTool.get(name)
        return PathTool.path(
            os.path.join(PATH_STATISTICS.FOLDER, j((name, FILE.EXTENSION.PNG), "."))
        )


class PATH_INDICATIONS:
    NAME: str = "indications"
    FOLDER: str = os.path.join(PATH_APP_DATA.FOLDER, NAME)

    CHILLER_DATA_NAME: str = "chiller"
    CHILLER_DATA_FOLDER: str = os.path.join(FOLDER, CHILLER_DATA_NAME)

    CHILLER_DATA_IMAGE_LAST: str = os.path.join(
        CHILLER_DATA_FOLDER, f"last.{FILE.EXTENSION.JPG}"
    )
    CHILLER_DATA_IMAGE_LAST_RESULT: str = os.path.join(
        CHILLER_DATA_FOLDER, f"last_result.{FILE.EXTENSION.JPG}"
    )

    @staticmethod
    def CHILLER_DATA_IMAGE_RESULT(
        datetime_string: str, temperature: float | None, indications: int
    ) -> str:
        name_list: list[str] = [str(indications)]
        if temperature is not None:
            name_list += [str(temperature)]
        name_list += [datetime_string]
        return os.path.join(
            PATH_INDICATIONS.CHILLER_DATA_FOLDER,
            f"{'_'.join(name_list)}.{FILE.EXTENSION.JPG}",
        )


class PATH_MOBILE_HELPER:
    NAME: str = "mobile helper"
    FOLDER: str = os.path.join(PATH_APP_DATA.FOLDER, NAME)

    QR_CODE_NAME: str = "qr code"
    QR_CODE_FOLDER: str = os.path.join(FOLDER, QR_CODE_NAME)

    INCOME_IMAGES_NAME: str = "income images"
    INCOME_IMAGES_FOLDER: str = os.path.join(FOLDER, INCOME_IMAGES_NAME)

    FILES_NAME: str = "files"
    FILES_FOLDER: str = os.path.join(FOLDER, FILES_NAME)

    TIME_TRACKING_REPORT: str = "time tracking report"
    TIME_TRACKING_REPORT_FOLDER: str = os.path.join(FOLDER, TIME_TRACKING_REPORT)


class PATH_POLIBASE_APP_DATA:
    NAME: str = "polibase"
    FOLDER: str = os.path.join(PATH_APP_DATA.FOLDER, NAME)
    PERSON_CARD_REGISTRY_FOLDER: str = os.path.join(FOLDER, "person card folder")

    SERVICE_FOLDER_PATH: str = os.path.join(
        PATH_FACADE.VALUE, f"{NAME}{FACADE.SERVICE_FOLDER_SUFFIX}"
    )

    class SETTINGS:
        MAIN: str = "polibase_main_settings.vbs"
        TEST: str = "polibase_test_settings.vbs"


class PATH_USER:
    NAME: str = "homes"
    HOME_FOLDER: str = os.path.join(AD.PATH_ROOT, NAME)
    HOME_FOLDER_FULL: str = os.path.join(AD.PATH_ROOT, NAME)

    @staticmethod
    def private_folder(login: str) -> str:
        return os.path.join(PATH_USER.HOME_FOLDER, login)

    @staticmethod
    def get_document_name(user_name: str, login: str | None = None) -> str:
        return (
            PATH_IT.get_new_employee_path(user_name)
            + (f" ({login})" if login else "")
            + ".docx"
        )


class PATH_POLIBASE:
    NAME: str = Hosts.POLIBASE.ALIAS
    DATA_FOLDER_NAME: str = "Data"
    PERSON_FOLDER_NAME: str = "PERSONS"
    TEST_SUFFIX: str = "_test"
    PERSON_CARD_REGISTRY_FOLDER: str = (
        PATH_POLIBASE_APP_DATA.PERSON_CARD_REGISTRY_FOLDER
    )

    @staticmethod
    def person_folder(pin: int | None = None, test: bool = False) -> str:
        root: str = PATH_POLIBASE.NAME
        if test:
            if root.find(".") != -1:
                root_parts: list[str] = root.split(".")
                root_parts[0] += PATH_POLIBASE.TEST_SUFFIX
                root = j(root_parts, ".")
            else:
                root += PATH_POLIBASE.TEST_SUFFIX
        return j(
            (
                r"\\",
                os.path.join(
                    root,
                    PATH_POLIBASE.DATA_FOLDER_NAME,
                    PATH_POLIBASE.PERSON_FOLDER_NAME,
                    "" if n(pin) else str(pin),
                ),
            )
        )


class PATH_WS:
    NAME: str = j(("WS", FACADE.SERVICE_FOLDER_SUFFIX))
    PATH: str = os.path.join(PATH_FACADE.VALUE, NAME)


class PATHS:
    SHARE: PATH_SHARE = PATH_SHARE()
    SCAN: PATH_SCAN = PATH_SCAN()
    MEDICAL_DATA: PATH_MEDICAL_DATA = PATH_MEDICAL_DATA()
    SCAN_TEST = PATH_SCAN_TEST()
    SCAN_SOURCE = PATH_SCAN_SOURCE()
    SCAN_RESULT = PATH_SCAN_RESULT()
    WS_816_SCAN: PATH_WS_816_SCAN = PATH_WS_816_SCAN()
    OMS: PATH_OMS = PATH_OMS()
    IT: PATH_IT = PATH_IT()
    USER: PATH_USER = PATH_USER()
    POLIBASE: PATH_POLIBASE = PATH_POLIBASE()
    POLIBASE_APP_DATA: PATH_POLIBASE_APP_DATA = PATH_POLIBASE_APP_DATA()
    WS: PATH_WS = PATH_WS()
    DOCS: PATH_DOCS = PATH_DOCS()
    FONTS: PATH_FONTS = PATH_FONTS()
    MOBILE_HELPER: PATH_MOBILE_HELPER = PATH_MOBILE_HELPER()
    APP_DATA: PATH_APP_DATA = PATH_APP_DATA()
    INDICATIONS: PATH_INDICATIONS = PATH_INDICATIONS()
    STATISTICS: PATH_STATISTICS = PATH_STATISTICS()
    FACADE: PATH_FACADE = PATH_FACADE()
    DATA_STORAGE: PATH_DATA_STORAGE = PATH_DATA_STORAGE()
    BUILD: PATH_BUILD = PATH_BUILD()



from enum import Enum, auto

class ZABBIX:
    
    class Commands(Enum):
        
        get_host_list = auto() 
        get_item_list = auto() 
        get_value_list = auto() 
        send_value = auto()


class FILE:
    class EXTENSION:
        EXCEL_OLD: str = "xls"
        EXCEL_NEW: str = "xlsx"
        JPEG: str = "jpeg"
        JPG: str = "jpg"
        PNG: str = "png"
        PDF: str = "pdf"
        TXT: str = "txt"
        JSON: str = "json"
        PYTHON: str = "py"
        BASE64: str = "base64"
        EXE: str = "exe"
        ARCHIVE: str = "zip"
        TRUE_TYPE_FONT: str = "ttf"
        WHEEL: str = "whl"


class RPC:
    PING_COMMAND: str = "__ping__"
    EVENT_COMMAND: str = "__event__"
    SUBSCRIBE_COMMAND: str = "__subscribe__"

    @staticmethod
    def PORT(add: int = 0) -> int:
        return 50051 + add

    TIMEOUT: int | None = None
    TIMEOUT_FOR_PING: int = 20
    LONG_OPERATION_DURATION: int | None = None


from pih.tools import j, js, jp
from pih.consts import CONST


class ADDRESSES:

    REMOTE_PREFIX: str = "remote"
    SITE_NAME: str = "pacifichosp"
    SITE_ADDRESS: str = jp((SITE_NAME, "com"))
    EMAIL_SERVER_ADDRESS: str = jp(("mail", SITE_ADDRESS))
    RECEPTION_NAME: str = "reception"
    CALL_CENTRE: str = "callcentre"
    ADD_EMAIL_NAME: str = "add_email"
    RECEPTION_LOGIN: str = jp((RECEPTION_NAME, SITE_NAME))

    WIKI_SITE_NAME: str = "wiki"
    WIKI_SITE_ADDRESS: str = WIKI_SITE_NAME
    OMS_SITE_NAME: str = "oms"
    OMS_SITE_ADDRESS: str = OMS_SITE_NAME
    API_SITE_ADDRESS: str = jp(("api", SITE_ADDRESS))
    BITRIX_SITE_URL: str = "bitrix.cmrt.ru"

    ZABBIX_SITE_NAME: str = "zabbix"
    ZABBIX_SITE: str = ""
    ZABBIX_SITE_INTERNAL: str = j((ZABBIX_SITE_NAME, CONST.SPLITTER, 8080))
    ZABBIX_SITE_REMOTE: str = j((jp((ZABBIX_SITE_NAME, SITE_ADDRESS)), CONST.SPLITTER, 58080))


class EMAIL_COLLECTION:
    MAIL_RU_NAME: str = "mail.ru"
    MAIL_RU_DAEMON: str = jp(("mailer-daemon@corp", MAIL_RU_NAME))
    MAIL_RU_IMAP_SERVER: str = jp(("imap", MAIL_RU_NAME))

    NAS: str = j(("nas", ADDRESSES.SITE_ADDRESS), CONST.EMAIL_SPLITTER)
    IT: str = j(("it", ADDRESSES.SITE_ADDRESS), CONST.EMAIL_SPLITTER)
    RECEPTION: str = j(
        (ADDRESSES.RECEPTION_NAME, ADDRESSES.SITE_ADDRESS), CONST.EMAIL_SPLITTER
    )
    CALL_CENTRE: str = j(
        (ADDRESSES.CALL_CENTRE, ADDRESSES.SITE_ADDRESS), CONST.EMAIL_SPLITTER
    )
    ADD_EMAIL: str = j(
        (ADDRESSES.ADD_EMAIL_NAME, ADDRESSES.SITE_ADDRESS), CONST.EMAIL_SPLITTER
    )
    EXTERNAL: str = j(
        ("mail.", ADDRESSES.SITE_NAME, CONST.EMAIL_SPLITTER, MAIL_RU_NAME)
    )

    EXTERNAL_SERVER: str = MAIL_RU_IMAP_SERVER



from pih.consts.rpc import RPC
from pih.consts.hosts import Hosts
from pih.consts.service import ServiceRoleBase
from pih.collections.service import ServiceDescription
from pih.consts.service_commands import ServiceCommands
from ipih import SERVICE_ADMIN_HOST_NAME, SERVICE_ADMIN_GRPC_PORT

class ServiceRoles(ServiceRoleBase):

    SERVICE_ADMIN = ServiceDescription(
        name="ServiceAdmin",
        host=SERVICE_ADMIN_HOST_NAME,
        port=RPC.PORT(SERVICE_ADMIN_GRPC_PORT),
        commands=(
            ServiceCommands.on_service_starts,
            ServiceCommands.on_service_stops,
            ServiceCommands.get_service_information_list,
            ServiceCommands.heart_beat,
        ),
    )

    EVENT = ServiceDescription(
        name="Event",
        commands=(ServiceCommands.send_log_message, ServiceCommands.send_event),
    )

    DS = ServiceDescription(
        name="DataSource",
        commands=(
            ServiceCommands.register_polibase_person_information_quest,
            ServiceCommands.search_polibase_person_information_quests,
            ServiceCommands.update_polibase_person_information_quest,
            #
            ServiceCommands.update_polibase_person_visit_to_data_stogare,
            ServiceCommands.search_polibase_person_visits_in_data_storage,
            #
            ServiceCommands.register_polibase_person_visit_notification,
            ServiceCommands.search_polibase_person_visit_notifications,
            #
            ServiceCommands.register_delayed_message,
            ServiceCommands.search_delayed_messages,
            ServiceCommands.update_delayed_message,
            #
            ServiceCommands.get_settings_value,
            ServiceCommands.set_settings_value,
            #
            ServiceCommands.search_polibase_person_notification_confirmation,
            ServiceCommands.update_polibase_person_notification_confirmation,
            #
            ServiceCommands.get_storage_value,
            ServiceCommands.set_storage_value,
            #
            ServiceCommands.get_ogrn_value,
            ServiceCommands.get_fms_unit_name,
            #
            ServiceCommands.register_chiller_indications_value,
            ServiceCommands.register_ct_indications_value,
            ServiceCommands.get_last_ct_indications_value_container_list,
            ServiceCommands.get_last_сhiller_indications_value_container_list,
            #
            ServiceCommands.get_gkeep_item_list_by_any,
            ServiceCommands.add_gkeep_item,
            #
            ServiceCommands.register_event,
            ServiceCommands.get_event,
            ServiceCommands.remove_event,
            #
            ServiceCommands.execute_data_source_query,
            ServiceCommands.joke,
            ServiceCommands.get_event_count,
        ),
    )

    AD = ServiceDescription(
        name="ActiveDirectory",
        commands=(
            ServiceCommands.authenticate,
            ServiceCommands.check_user_exists_by_login,
            ServiceCommands.get_user_by_full_name,
            ServiceCommands.get_users_by_name,
            ServiceCommands.get_user_by_login,
            ServiceCommands.get_user_by_telephone_number,
            ServiceCommands.get_template_users,
            ServiceCommands.get_containers,
            ServiceCommands.get_user_list_by_job_position,
            ServiceCommands.get_user_list_by_group,
            ServiceCommands.get_printer_list,
            ServiceCommands.get_computer_list,
            ServiceCommands.get_computer_description_list,
            ServiceCommands.get_workstation_list_by_user_login,
            ServiceCommands.get_user_by_workstation,
            ServiceCommands.create_user_by_template,
            ServiceCommands.set_user_telephone_number,
            ServiceCommands.set_user_password,
            ServiceCommands.set_user_status,
            ServiceCommands.remove_user,
            ServiceCommands.drop_user_cache,
            ServiceCommands.drop_workstaion_cache,
            ServiceCommands.get_user_list_by_property,
        ),
    )

    RECOGNIZE = ServiceDescription(
        name="Recognize",
        commands=(
            ServiceCommands.get_barcode_list_information,
            ServiceCommands.document_type_exists,
            ServiceCommands.recognize_document,
        ),
    )

    FILE_WATCHDOG = ServiceDescription(
        name="FileWatchdog", commands=(ServiceCommands.listen_for_new_files,)
    )

    """
    FILE_ORGANIZER = ServiceDescription(
        name="FileOrganizer",
        description="File organizer service",
        host=HOSTS.WS255.NAME,
    )
    """

    MAIL = ServiceDescription(
        name="Mail",
        commands=(
            ServiceCommands.check_email_accessibility,
            ServiceCommands.send_email,
            ServiceCommands.get_email_information,
        ),
    )

    DOCS = ServiceDescription(
        name="Docs",
        commands=(
            ServiceCommands.get_inventory_report,
            ServiceCommands.create_user_document,
            ServiceCommands.save_time_tracking_report,
            ServiceCommands.create_barcodes_for_inventory,
            ServiceCommands.create_barcode_for_polibase_person,
            ServiceCommands.create_qr_code,
            ServiceCommands.check_inventory_report,
            ServiceCommands.save_inventory_report_item,
            ServiceCommands.close_inventory_report,
            ServiceCommands.create_note,
            ServiceCommands.get_note,
            ServiceCommands.get_note_list_by_label,
            ServiceCommands.create_statistics_chart,
            ServiceCommands.save_xlsx,
            ServiceCommands.drop_note_cache,
        ),
    )

    MARK = ServiceDescription(
        name="Mark",
        commands=(
            ServiceCommands.get_free_mark_list,
            ServiceCommands.get_temporary_mark_list,
            ServiceCommands.get_mark_person_division_list,
            ServiceCommands.get_time_tracking,
            ServiceCommands.get_mark_list,
            ServiceCommands.get_mark_by_tab_number,
            ServiceCommands.get_mark_by_person_name,
            ServiceCommands.get_free_mark_group_statistics_list,
            ServiceCommands.get_free_mark_list_by_group_id,
            ServiceCommands.get_owner_mark_for_temporary_mark,
            ServiceCommands.get_mark_list_by_division_id,
            ServiceCommands.set_full_name_by_tab_number,
            ServiceCommands.set_telephone_by_tab_number,
            ServiceCommands.check_mark_free,
            ServiceCommands.create_mark,
            ServiceCommands.make_mark_as_free_by_tab_number,
            ServiceCommands.make_mark_as_temporary,
            ServiceCommands.remove_mark_by_tab_number,
            ServiceCommands.door_command,
        ),
    )

    POLIBASE = ServiceDescription(
        name="Polibase",
        commands=(
            ServiceCommands.get_polibase_person_by_pin,
            ServiceCommands.get_polibase_persons_by_pin,
            ServiceCommands.get_polibase_persons_by_telephone_number,
            ServiceCommands.get_polibase_persons_by_name,
            ServiceCommands.get_polibase_persons_by_card_registry_folder_name,
            ServiceCommands.get_polibase_person_registrator_by_pin,
            ServiceCommands.get_polibase_person_pin_list_with_old_format_barcode,
            #
            ServiceCommands.get_polibase_persons_pin_by_visit_date,
            #
            ServiceCommands.search_polibase_person_visits,
            ServiceCommands.get_polibase_person_visits_last_id,
            #
            ServiceCommands.set_polibase_person_card_folder_name,
            ServiceCommands.set_polibase_person_email,
            ServiceCommands.set_barcode_for_polibase_person,
            ServiceCommands.check_polibase_person_card_registry_folder_name,
            ServiceCommands.set_polibase_person_telephone_number,
            ServiceCommands.get_polibase_person_operator_by_pin,
            ServiceCommands.get_polibase_person_by_email,
            #
            ServiceCommands.execute_polibase_query,
            ServiceCommands.update_person_change_date,
            ServiceCommands.get_polibase_person_pin_by_login,
            ServiceCommands.get_polibase_person_user_login_and_worstation_name_pair_list,
            ServiceCommands.get_bonus_list,
        ),
    )

    MESSAGE_RECEIVER = ServiceDescription(name="MessageReceiver")

    WEB_SERVER = ServiceDescription(name="WebServer")

    CHECKER = ServiceDescription(
        name="Checker", commands=(ServiceCommands.get_resource_status_list,)
    )

    AUTOMATION = ServiceDescription(name="Automation")

    MOBILE_HELPER = ServiceDescription(name="MobileHelper", standalone_name="mio")

    POLIBASE_AUTOMATION = ServiceDescription(
        name="PolibaseAutomation", description="Polibase automation service"
    )

    MESSAGE_QUEUE = ServiceDescription(
        name="MessageQueue",
        commands=(ServiceCommands.add_message_to_queue,),
    )

    NOTIFICATION_AUTOMATION = ServiceDescription(name="NotificationAutomation")

    REVIEW_AUTOMATION = ServiceDescription(
        name="ReviewNotification"
    )

    WS = ServiceDescription(
        name="WS",
        commands=(
            ServiceCommands.reboot,
            ServiceCommands.shutdown,
            ServiceCommands.send_message_to_user_or_workstation,
            ServiceCommands.kill_process,
        ),
    )

    BACKUP = ServiceDescription(
        name="Backup",
        commands=(
            ServiceCommands.robocopy_start_job,
            ServiceCommands.robocopy_get_job_status_list,
        ),
        standalone_name="bck",
    )

    PRINTER = ServiceDescription(
        name="Printer",
        commands=(ServiceCommands.printers_report, ServiceCommands.printer_snmp_call),
    )
    
    
    ZABBIX = ServiceDescription(
        name="Zabbix",
    )
    
    IOTDevices = ServiceDescription(
        name="IOTDevices",
    )

    POLIBASE_DATABASE = ServiceDescription(
        name="PolibaseDatabase",
        commands=(ServiceCommands.create_polibase_database_backup,),
    )

    SSH = ServiceDescription(
        name="SSH",
        commands=(
            ServiceCommands.execute_ssh_command,
            ServiceCommands.get_certificate_information,
            ServiceCommands.get_unix_free_space_information_by_drive_name,
            ServiceCommands.mount_facade_for_linux_host,
        ),
    )

    """
    POLIBASE_PERSON_INFORMATION_QUEST = ServiceDescription(
        name="PolibasePersonInformationQuest",
        description="Polibase Person Information Quest service",
        host=Hosts.BACKUP_WORKER.NAME,
        commands=(ServiceCommands.start_polibase_person_information_quest,),
    )
    """

    WS735 = ServiceDescription(
        name="ws735",
        description="ws-735 service",
        host=Hosts.WS735.NAME,
        commands=(ServiceCommands.print_image,)
    )

    RECOGNIZE_TEST = ServiceDescription(
        name="RecognizeTest",
        description="Recognize test service",
        host=Hosts.WS255.NAME,
        auto_start=False,
        auto_restart=False,
        visible_for_admin=False,
    )

    """INDICATIONS = ServiceDescription(
        name="Indications",
        description="Indications service",
        host=HOSTS.WS255.NAME,
    )"""

    MEDICAL_AUTOMATION = ServiceDescription(
        name="MedicalAutomation",
        description="Medical Automation service",
        host=Hosts.BACKUP_WORKER.NAME,
    )

    REGISTRATOR_HELPER = ServiceDescription(name="RegistratorHelper")

    DEVELOPER = ServiceDescription(
        name="Developer",
        description="Developer service",
        host=Hosts.DEVELOPER.NAME,
        port=RPC.PORT(1),
        visible_for_admin=False,
        auto_start=False,
        auto_restart=False,
        commands=(ServiceCommands.test,),
    )



from enum import Enum, auto, IntEnum

from pih.tools import j
from pih.consts import *
from pih.consts.names import *
from pih.consts.paths import *
from pih.collections import (
    ActionDescription,
    ResourceDescription,
    MedicalResearchType,
    SiteResourceDescription,
    MinIntStorageVariableHolder,
    ResourceDescriptionDelegated,
    OrderedNameCaptionDescription,
    IconedOrderedNameCaptionDescription,
)
from pih.consts.password import *
from pih.consts.date_time import *
from pih.consts.service_commands import *

VERSION: str = "0.36.14"

class DATA:
    # deprecated
    class EXTRACTOR:
        USER_NAME_FULL: str = "user_name_full"
        USER_NAME: str = "user_name"
        AS_IS: str = "as_is"

    class FORMATTER(Enum):
        MY_DATETIME = "my_datetime"
        MY_DATE = "my_date"
        CHILLER_INDICATIONS_VALUE_INDICATORS = "chiller_indications_value_indicators"
        CHILLER_FILTER_COUNT = "chiller_filter_count"


class INDICATIONS:
    class CHILLER:
        ACTUAL_VALUES_TIME_DELTA_IN_MINUTES: int = 5

        INDICATOR_NAME: list[str] = [
            "Активный сигнал тревоги",
            "Работает нагреватель",
            "Замораживание включено",
            "Работает вентилятор конденсатора",
            "Работает насос/вытяжной вентилятор",
            "Работает компрессор",
        ]

        INDICATOR_EMPTY_DISPLAY: int = -1

    class MRI:
        # hours
        PERIOD: int = 2


class BARCODE:
    CODE128: str = "code128"
    I25: str = "i25"


class FONT:
    FOR_PDF: str = "DejaVuSerif"


class SessionFlags(IntEnum):
    CLI = 512


class URLS:
    PYPI: str = "https://pypi.python.org/pypi/pih/json"


class EmailVerificationMethods(IntEnum):
    NORMAL = auto()
    ABSTRACT_API = auto()
    DEFAULT = ABSTRACT_API


class ROBOCOPY:
    ERROR_CODE_START: int = 8

    STATUS_CODE: dict[int, str] = {
        0: "No errors occurred, and no copying was done. The source and destination directory trees are completely synchronized.",
        1: "One or more files were copied successfully (that is, new files have arrived).",
        2: "Some Extra files or directories were detected. No files were copied Examine the output log for details.",
        4: "Some Mismatched files or directories were detected. Examine the output log. Housekeeping might be required.",
        8: "Some files or directories could not be copied (copy errors occurred and the retry limit was exceeded). Check these errors further.",
        16: "Serious error. Robocopy did not copy any files. Either a usage error or an error due to insufficient access privileges on the source or destination directories.",
        3: "Some files were copied. Additional files were present. No failure was encountered.",
        5: "Some files were copied. Some files were mismatched. No failure was encountered.",
        6: "Additional files and mismatched files exist. No files were copied and no failures were encountered. This means that the files already exist in the destination directory",
        7: "Files were copied, a file mismatch was present, and additional files were present.",
    }


class CHARSETS:
    WINDOWS_ALTERNATIVE: str = "CP866"
    UTF8: str = "utf-8"

class WINDOWS:

    NAME: str = "Windows"

    class ENVIROMENT_VARIABLES:
        PATH: str = "PATH"

    ENVIROMENT_COMMAND: str = "$Env"

    class CHARSETS:
        ALTERNATIVE: str = CHARSETS.WINDOWS_ALTERNATIVE

    class SERVICES:
        WIA: str = (
            "stisvc"  # Обеспечивает службы получения изображений со сканеров и цифровых камер
        )
        TASK_SCHEDULER: str = "schtasks"

    class PROCESSES:
        POWER_SHELL_REMOTE_SESSION: str = "wsmprovhost.exe"

    class PORT:
        SMB: int = 445


class CONST(DATE_TIME):
    
    EMAIL_SPLITTER: str = "@"
    ISOLATED_ARG_NAME: str = "isolated"
    ARGUMENT_PREFIX: str = "--"
    SPLITTER: str = ":"
    UNKNOWN: str = "?"

    class PORT:
        HTTP: int = 80
        SMTP: int = 587
        IMAP: int = 993
        SNMP: int = 161

    class FILES:
        SECTION: str = "Мобильные файлы"

    class NOTES:
        SECTION: str = "Мобильные заметки"

    class JOURNALS:
        SECTION: str = "Журналы"

    # in seconds
    HEART_BEAT_PERIOD: int = 60

    NEW_LINE: str = "\n"

    class TEST:
        WORKSTATION_MAME: str = Hosts.DEVELOPER.NAME
        USER: str | None = "nak"
        EMAIL_ADDRESS: str | None = "nak@pacifichosp.com"
        PIN: int = 100310
        NAME: str = "test"

    GROUP_PREFIX: str = "group:"

    SITE_PROTOCOL: str = "https://"
    UNTRUST_SITE_PROTOCOL: str = "http://"

    INTERNATIONAL_TELEPHONE_NUMBER_PREFIX: str = "7"
    TELEPHONE_NUMBER_PREFIX: str = f"+{INTERNATIONAL_TELEPHONE_NUMBER_PREFIX}"
    INTERNAL_TELEPHONE_NUMBER_PREFIX: str = "тел."

    class CACHE:
        class TTL:
            # in seconds
            WORKSTATIONS: int = 60
            USERS: int = 300

    class ERROR:
        class WAPPI:
            PROFILE_NOT_PAID: int = 402

    class TIME_TRACKING:
        REPORT_DAY_PERIOD_DEFAULT: int = 15

    class MESSAGE:
        class WHATSAPP:
            SITE_NAME: str = "https://wa.me/"
            SEND_MESSAGE_TO_TEMPLATE: str = SITE_NAME + "{}?text={}"
            GROUP_SUFFIX: str = "@g.us"

            class GROUP(Enum):
                # IT: str = "120363163438805316@g.us"
                PIH_CLI = "120363163438805316@g.us"
                REGISTRATOR_CLI = "120363212130686795@g.us"
                RD = "79146947050-1595848245@g.us"
                MAIN = "79644300470-1447044803@g.us"
                EMAIL_CONTROL = "120363159605715569@g.us"
                CT_INDICATIONS = "120363084280723039@g.us"
                DOCUMENTS_WORK_STACK = "120363115241877592@g.us"
                REGISTRATION_AND_CALL = "79242332784-1447983812@g.us"
                DOCUMENTS_WORK_STACK_TEST = "120363128816931482@g.us"
                CONTROL_SERVICE_INDICATIONS = "120363159210756301@g.us"
                SCANNED_DOCUMENT_HELPER_CLI = "120363220286578760@g.us"

            class WAPPI:

                DATETIME_FORMAT: str = "%Y-%m-%dT%H:%M:%SZ"
                NAME: str = "Wappi"
                DESCRIPTION: str = "Сервис по отправке сообщений"

                PROFILE_SUFFIX: str = "profile_id="
                URL_API: str = "https://wappi.pro/api"
                URL_API_SYNC: str = f"{URL_API}/sync"
                URL_MESSAGE: str = f"{URL_API_SYNC}/message"
                STATUS: str = j(
                    (j((URL_API_SYNC, "get", "status"), "/"), "?", PROFILE_SUFFIX)
                )
                URL_SEND_MESSAGE: str = f"{URL_MESSAGE}/send?{PROFILE_SUFFIX}"
                URL_SEND_VIDEO: str = f"{URL_MESSAGE}/video/send?{PROFILE_SUFFIX}"
                URL_SEND_IMAGE: str = f"{URL_MESSAGE}/img/send?{PROFILE_SUFFIX}"
                URL_SEND_DOCUMENT: str = f"{URL_MESSAGE}/document/send?{PROFILE_SUFFIX}"
                URL_SEND_LIST_MESSAGE: str = f"{URL_MESSAGE}/list/send?{PROFILE_SUFFIX}"
                URL_SEND_BUTTONS_MESSAGE: str = (
                    f"{URL_MESSAGE}/buttons/send?{PROFILE_SUFFIX}"
                )
                URL_GET_MESSAGES: str = f"{URL_MESSAGE}s/get?{PROFILE_SUFFIX}"
                URL_GET_STATUS: str = f"{URL_API_SYNC}/get/status?{PROFILE_SUFFIX}"
                CONTACT_SUFFIX: str = "@c.us"

                class Profiles(Enum):
                    IT = "e6706eaf-ae17"
                    CALL_CENTRE = "285c71a4-05f7"
                    MARKETER = "c31db01c-b6d6"
                    DEFAULT = CALL_CENTRE

                AUTHORIZATION: dict[Profiles, str] = {
                    Profiles.IT: "6b356d3f53124af3078707163fdaebca3580dc38",
                    Profiles.MARKETER: "6b356d3f53124af3078707163fdaebca3580dc38",
                    Profiles.CALL_CENTRE: "7d453de6fc17d3e6816b0abc46f2b192822130f5",
                }

    class SERVICE:
        NAME: str = "service"

    class VALENTA:
        NAME: str = "valenta"
        PROCESS_NAME: str = "Vlwin"

    class POWERSHELL:
        NAME: str = "powershell"

    class PSTOOLS:
        NAME: str = "pstools"
        PS_EXECUTOR: str = "psexec"
        PS_KILL_EXECUTOR: str = "pskill"
        PS_PING: str = "psping"

        COMMAND_LIST: list[str] = [
            PS_KILL_EXECUTOR,
            "psfile",
            "psgetsid",
            "psinfo",
            "pslist",
            "psloggedon",
            "psloglist",
            "pspasswd",
            PS_PING,
            "psservice",
            "psshutdown",
            "pssuspend",
        ]

        NO_BANNER: str = "-nobanner"
        ACCEPTEULA: str = "-accepteula"
    

    class MSG:
        NAME: str = "msg"
        EXECUTOR: str = NAME

    class BARCODE_READER:
        PREFIX: str = "("
        SUFFIX: str = ")"

    class NAME_POLICY:
        PARTS_LIST_MIN_LENGTH: int = 3
        PART_ITEM_MIN_LENGTH: int = 3

    HOST = Hosts

    class CARD_REGISTRY:
        PLACE_NAME: dict[str, str] = {"Т": "Приёмное отделение", "П": "Поликлиника"}
        PLACE_CARD_HOLDER_MAPPER: dict[str, str] = {"Т": "М-Я", "П": "А-Л"}
        MAX_CARD_PER_FOLDER: int = 60
        SUITABLE_FOLDER_NAME_SYMBOL = ("!", " ")

    class VISUAL:
        YES: str = "✅"
        NO: str = "❌"
        WARNING: str = "⚠️"
        WAIT: str = "⏳"
        NOTIFICATION: str = "🔔"
        ROBOT: str = "🤖"
        SAD: str = "😔"
        GOOD: str = YES
        ERROR: str = NO
        ORANGE_ROMB: str = "🔸"
        BLUE_ROMB: str = "🔹"
        TASK: str = "✳️"
        EYE: str = "👁️"
        HAND_INDICATE: str = "👉"
        HAND_DOWN: str = "👇"
        HAND_UP: str = "☝️"
        INFORMATION: str = "ℹ️"
        QUESTION: str = "❔"

        NUMBER_SYMBOLS: list[str] = [
            "0️⃣",
            "1️⃣",
            "2️⃣",
            "3️⃣",
            "4️⃣",
            "5️⃣",
            "6️⃣",
            "7️⃣",
            "8️⃣",
            "9️⃣",
            "🔟",
        ]

        TEMPERATURE_SYMBOL: str = "°C"

        ARROW: str = "➜"

        BULLET: str = "•"


class MATERIALIZED_RESOURCES:
    NAME: str = "MATERIALIZED_RESOURCES"
    ALIAS: str = "MR"

    class Types(Enum):

        CHILLER_FILTER = MinIntStorageVariableHolder(
            "CHF", description="Фильтры для чиллера", min_value=2
        )

        """
        OPTICAL_DISK_IN_STOCK = MinIntStorageVariableHolder(
            "ODS",
            description="Оптические диски для записи исследований на складе",
            min_value=50,
        )

        OPTICAL_DISK_IN_USE = MinIntStorageVariableHolder(
            "ODU",
            description="Оптические диски для записи исследований в пользовании",
            min_value=10,
        )
        """


class MedicalResearchTypes(Enum):
    MRI = MedicalResearchType(("Магнитно-резонансная томография",), "МРТ")
    CT = MedicalResearchType(("Компьютерная томография",), "КТ")
    ULTRASOUND = MedicalResearchType(("ультразвуковая допплерография",), "УЗИ")


from pih.consts.addresses import ADDRESSES


class RESOURCES:
    class DESCRIPTIONS:
        INTERNET: ResourceDescription = ResourceDescription(
            "77.88.55.242", "Интернет соединение"
        )

        VPN_PACS_SPB: ResourceDescriptionDelegated = ResourceDescriptionDelegated(
            "192.168.5.3", "VPN соединение для PACS SPB", (2, 100, 5), Hosts.WS255.NAME
        )

        PACS_SPB: ResourceDescriptionDelegated = ResourceDescriptionDelegated(
            "10.76.12.124:4242", "Соединение PACS SPB", (2, 100, 5), Hosts.WS255.NAME
        )

        POLIBASE1: ResourceDescription = ResourceDescription(
            Hosts.POLIBASE1.NAME,
            "Polibase",
            inaccessibility_check_values=(2, 10000, 15),
        )

        POLIBASE2: ResourceDescription = ResourceDescription(
            Hosts.POLIBASE2.NAME,
            "Polibase reserved",
            inaccessibility_check_values=(2, 10000, 15),
        )

        POLIBASE: ResourceDescription = ResourceDescription(
            Hosts.POLIBASE.ALIAS,
            "Polibase",
            inaccessibility_check_values=(2, 10000, 15),
        )

        SITE_LIST: list[SiteResourceDescription] = [
            SiteResourceDescription(
                ADDRESSES.SITE_ADDRESS,
                f"Сайт компании: {ADDRESSES.SITE_ADDRESS}",
                inaccessibility_check_values=(1, 20, 15),
                check_certificate_status=True,
                check_free_space_status=True,
                driver_name="/dev/mapper/centos-root",
            ),
            SiteResourceDescription(
                ADDRESSES.EMAIL_SERVER_ADDRESS,
                f"Сайт корпоративной почты: {ADDRESSES.EMAIL_SERVER_ADDRESS}",
                check_certificate_status=True,
                check_free_space_status=True,
                driver_name="/dev/mapper/centos_tenant26--02-var",
            ),
            SiteResourceDescription(
                ADDRESSES.API_SITE_ADDRESS,
                f"Api сайта {ADDRESSES.SITE_ADDRESS}: {ADDRESSES.API_SITE_ADDRESS}",
                check_certificate_status=True,
                check_free_space_status=False,
            ),
            SiteResourceDescription(
                ADDRESSES.BITRIX_SITE_URL, f"Сайт ЦМРТ24: {ADDRESSES.BITRIX_SITE_URL}"
            ),
            SiteResourceDescription(
                ADDRESSES.OMS_SITE_ADDRESS,
                f"Внутренний сайт омс: {ADDRESSES.OMS_SITE_ADDRESS}",
                internal=True,
            ),
            SiteResourceDescription(
                ADDRESSES.WIKI_SITE_ADDRESS,
                f"Внутренний сайт Вики: {ADDRESSES.WIKI_SITE_ADDRESS}",
                internal=True,
            ),
        ]


class CheckableSections(IntEnum):
    RESOURCES = auto()
    WS = auto()
    PRINTERS = auto()
    INDICATIONS = auto()
    BACKUPS = auto()
    VALENTA = auto()
    SERVERS = auto()
    MATERIALIZED_RESOURCES = auto()
    TIMESTAMPS = auto()
    DISKS = auto()
    POLIBASE = auto()

    @staticmethod
    def all():
        return [item for item in CheckableSections]


class MarkType(IntEnum):
    NORMAL = auto()
    FREE = auto()
    GUEST = auto()
    TEMPORARY = auto()


class PolibasePersonInformationQuestStatus(IntEnum):
    UNKNOWN = -1
    GOOD = 0
    EMAIL_IS_EMPTY = 1
    EMAIL_IS_WRONG = 2
    EMAIL_IS_NOT_ACCESSABLE = 4


class ResourceInaccessableReasons(Enum):
    CERTIFICATE_ERROR = "Ошибка проверки сертификата"
    SERVICE_UNAVAILABLE = "Ошибка 503: Сервис недоступен"


class PolibasePersonReviewQuestStep(IntEnum):
    BEGIN = auto()
    #
    ASK_GRADE = auto()
    ASK_FEEDBACK_CALL = auto()
    ASK_INFORMATION_WAY = auto()
    #
    COMPLETE = auto()


LINK_EXT = "lnk"


class CommandTypes(Enum):
    POLIBASE = (
        "Запрос к базе данный Polibase (Oracle)",
        "polibase",
        "полибейс",
        "oracle",
    )
    DATA_SOURCE = ("Запрос к базе данных DataSource (DS)", "ds")
    CMD = ("Консольную команду", "cmd")
    POWERSHELL = ("Powershell команду", "powershell")
    PYTHON = ("Скрипт Python", "py", "python")
    SSH = ("Команда SSH", "ssh")


class LogMessageChannels(IntEnum):
    BACKUP = auto()
    POLIBASE = auto()
    POLIBASE_BOT = auto()
    DEBUG = auto()
    DEBUG_BOT = auto()
    SERVICES = auto()
    SERVICES_BOT = auto()
    HR = auto()
    HR_BOT = auto()
    IT = auto()
    IT_BOT = auto()
    RESOURCES = auto()
    RESOURCES_BOT = auto()
    PRINTER = auto()
    POLIBASE_ERROR = auto()
    POLIBASE_ERROR_BOT = auto()
    CARD_REGISTRY = auto()
    CARD_REGISTRY_BOT = auto()
    NEW_EMAIL = auto()
    NEW_EMAIL_BOT = auto()
    TIME_TRACKING = auto()
    JOURNAL = auto()
    JOURNAL_BOT = auto()
    POLIBASE_DOCUMENT = auto()
    POLIBASE_DOCUMENT_BOT = auto()
    DEFAULT = DEBUG


class LogMessageFlags(IntEnum):
    NORMAL = 1
    ERROR = 2
    NOTIFICATION = 4
    DEBUG = 8
    SAVE = 16
    SILENCE = 32
    RESULT = 64
    WHATSAPP = 128
    ALERT = 256
    TASK = 512
    SAVE_ONCE = 1024
    SEND_ONCE = SAVE_ONCE | 2048
    DEFAULT = NORMAL


class SUBSCRIBTION_TYPE:
    ON_PARAMETERS: int = 1
    ON_RESULT: int = 2
    ON_RESULT_SEQUENTIALLY: int = 4


class WorkstationMessageMethodTypes(IntEnum):
    REMOTE = auto()
    LOCAL_MSG = auto()
    LOCAL_PSTOOL_MSG = auto()


class MessageTypes(IntEnum):
    WHATSAPP = auto()
    TELEGRAM = auto()
    WORKSTATION = auto()


class MessageStatuses(IntEnum):
    REGISTERED = 0
    COMPLETE = 1
    AT_WORK = 2
    ERROR = 3
    ABORT = 4


class SCAN:
    SPLITTER_DATA: str = "1"

    class Sources(Enum):
        POLICLINIC = ("poly", "Поликлиника", PATH_SCAN.VALUE)
        DIAGNOSTICS = (
            "diag",
            "Приёмное отделение",
            PATH_SCAN.VALUE,
        )
        TEST = ("test", "Тестовый", PATH_SCAN_TEST.VALUE)
        # Deprecated
        WS_816 = (
            "рисунок",
            "Дневной стационар",
            PATH_WS_816_SCAN.VALUE,
        )

        MEDICAL_DATA = (
            "рисунок",
            "Пересылка отсканированных документов с компьютера ws-816 (Дневной стационар)",
            PATH_MEDICAL_DATA.VALUE,
        )


class Actions(Enum):

    CHILLER_FILTER_CHANGING = ActionDescription(
        "CHILLER_FILTER_CHANGING",
        ("filter",),
        "Замена фильтра очистки воды",
        "Заменить фильтр очистки воды",
    )

    SWITCH_TO_EXTERNAL_WATER_SOURCE = ActionDescription(
        "SWITCH_TO_EXTERNAL_WATER_SOURCE",
        ("external_ws",),
        "Переход на внешнее (городское) водоснабжение",
        "Перейти на внешнее (городское) водоснабжение",
    )

    SWITCH_TO_INTERNAL_WATER_SOURCE = ActionDescription(
        "SWITCH_TO_INTERNAL_WATER_SOURCE",
        ("internal_ws",),
        "Переход на внутреннее водоснабжение",
        "Перейти на внутреннее водоснабжение",
    )

    VALENTA_SYNCHRONIZATION = ActionDescription(
        "VALENTA_SYNCHRONIZATION",
        (CONST.VALENTA.NAME, "валента"),
        "Синхронизация Валенты",
        "Совершить синхронизацию для Валенты",
        False,
        True,
        forcable=True,
    )

    TIME_TRACKING_REPORT = ActionDescription(
        "TIME_TRACKING_REPORT",
        ("tt", "урв"),
        "Отчеты по учёту рабочего времени",
        "Создать",
        False,
        False,
    )

    DOOR_OPEN = ActionDescription(
        "DOOR_OPEN",
        ("door_open",),
        "Открыть {name} дверь",
        "Открыть",
        False,
        False,
    )

    DOOR_CLOSE = ActionDescription(
        "DOOR_CLOSE",
        ("door_close",),
        "Закрыть {name} дверь",
        "Закрыть",
        False,
        False,
    )

    ATTACH_SHARED_DISKS = ActionDescription(
        "ATTACH_SHARED_DISKS",
        ("attach",),
        "Присоединить сетевые диски",
        "Присоединить",
        False,
        True,
    )

    ACTION = ActionDescription(
        "ACTION",
        ("action",),
        "Неспециализированное действие",
        None,
        False,
        True,
        forcable=False,
    )


class STATISTICS:
    class Types(Enum):
        CT = "CT"
        CT_DAY = "CT_DAY"
        CT_WEEK = "CT_WEEK"
        CT_MONTH = "CT_MONTH"
        CHILLER_FILTER = MATERIALIZED_RESOURCES.Types.CHILLER_FILTER.name
        MRI_COLDHEAD = "MRI_COLDHEAD"
        POLIBASE_DATABASE_DUMP = "POLIBASE_DATABASE_DUMP"
        POLIBASE_PERSON_REVIEW_NOTIFICATION = "POLIBASE_PERSON_REVIEW_NOTIFICATION"


class JournalType(tuple[int, OrderedNameCaptionDescription], Enum):
    GENERAL = (
        0,
        OrderedNameCaptionDescription("general", "Общий", order=1),
    )
    COMPUTER = (
        1,
        OrderedNameCaptionDescription("computer", "Компьютер", order=2),
    )
    MRI_CHILLER = (
        2,
        OrderedNameCaptionDescription("mri_chiller", "Чиллер МРТ", order=4),
    )
    MRI_GRADIENT_CHILLER = (
        3,
        OrderedNameCaptionDescription(
            "mri_gradient_chiller", "Чиллер градиентов МРТ", order=5
        ),
    )
    MRI_CLOSET_CHILLER = (
        4,
        OrderedNameCaptionDescription(
            "mri_closet_chiller", "Чиллер кабинета МРТ", order=6
        ),
    )
    CHILLER = (5, OrderedNameCaptionDescription("chiller", "Чиллер", order=3))
    COMMUNICATION_ROOM = (
        6,
        OrderedNameCaptionDescription(
            "communication_room", "Коммутационная комната", order=10
        ),
    )
    SERVER_ROOM = (
        7,
        OrderedNameCaptionDescription("server_room", "Серверная комната", order=9),
    )
    MRI_TECHNICAL_ROOM = (
        8,
        OrderedNameCaptionDescription(
            "mri_technical_room", "Техническая комната МРТ", order=7
        ),
    )
    MRI_PROCEDURAL_ROOM = (
        9,
        OrderedNameCaptionDescription(
            "mri_procedural_room", "Процедурная комната МРТ", order=9
        ),
    )
    PRINTER = (
        10,
        OrderedNameCaptionDescription("printer", "Принтер", order=11),
    )
    SERVER = (
        11,
        OrderedNameCaptionDescription("server", "Сервер", order=12),
    )
    OUTSIDE_SERVER = (
        12,
        OrderedNameCaptionDescription("outside_server", "Внешний сервер", order=13),
    )
    AGREEMENT = (
        13,
        OrderedNameCaptionDescription("agreement", "Договор и счет", order=14),
    )
    XRAY = (
        14,
        OrderedNameCaptionDescription("xray", "Рентген", order=15),
    )
    CASH_REGISTER = (
        15,
        OrderedNameCaptionDescription("cash_register", "Касса", order=16),
    )
    CT = (
        16,
        OrderedNameCaptionDescription("ct", "КТ", order=2),
    )


class Tags(tuple[int, IconedOrderedNameCaptionDescription], Enum):
    SERVICE = (
        1,
        IconedOrderedNameCaptionDescription(
            None, "Обслуживание", None, 4, CONST.VISUAL.GOOD
        ),
    )
    ERROR = (
        2,
        IconedOrderedNameCaptionDescription(
            None, "Ошибка", None, 2, CONST.VISUAL.ERROR
        ),
    )
    WARNING = (
        3,
        IconedOrderedNameCaptionDescription(
            None, "Внимание", None, 3, CONST.VISUAL.WARNING
        ),
    )
    NOTIFICATION = (
        4,
        IconedOrderedNameCaptionDescription(
            None, "Уведомление", None, 1, CONST.VISUAL.NOTIFICATION
        ),
    )
    TASK = (
        5,
        IconedOrderedNameCaptionDescription(
            None,
            "Задача",
            None,
            6,
            CONST.VISUAL.TASK,
        ),
    )
    INSPECTION = (
        6,
        IconedOrderedNameCaptionDescription(
            None,
            "Осмотр",
            None,
            5,
            CONST.VISUAL.EYE,
        ),
    )
    INFORMATION = (
        7,
        IconedOrderedNameCaptionDescription(
            None,
            "Информация",
            None,
            0,
            CONST.VISUAL.INFORMATION,
        ),
    )



import ipih
from pih.consts.service_commands import ServiceCommands

from dataclasses import dataclass, field
from typing import Any


@dataclass
class ServiceDescriptionBase:
    name: str | None = None
    host: str | None = None
    port: int | None = None
    service_path: str | None = None
    isolated: bool = False
    host_changeable: bool = True
    visible_for_admin: bool = True
    auto_start: bool = True
    auto_restart: bool = True
    run_from_system_account: bool = False
    python_executable_path: str | None = None
    version: str | None = None
    pih_version: str | None = None
    parameters: Any | None = None

    def __hash__(self) -> int:
        return hash(self.name)

    def __eq__(self, another):
        return (
            False
            if another is None
            else (
                self.name == another.name
                if isinstance(another, ServiceDescriptionBase)
                else self.name == another
            )
        )


@dataclass
class SubscribtionDescription:
    service_command: ServiceCommands | None = None
    type: int | None = None
    name: str | None = None


@dataclass
class Subscribtion(SubscribtionDescription):
    available: bool = False
    enabled: bool = False


@dataclass
class SubscribtionInformation(SubscribtionDescription):
    pass


@dataclass
class ServiceInformation(ServiceDescriptionBase):
    subscribtions: list[Subscribtion] = field(default_factory=list)
    pid: int = -1
    standalone: bool | None = None

    def __hash__(self) -> int:
        return hash(self.name)

    def __eq__(self, another: Any):
        return (
            False
            if another is None
            else (
                self.name == another.name
                if isinstance(another, ServiceDescriptionBase)
                else self.name == another
            )
        )


@dataclass
class ServiceDescription(ServiceDescriptionBase):
    description: str | None = None
    login: str | None = None
    password: str | None = None
    commands: tuple[ServiceCommands | str, ...] | None = None
    use_standalone: bool = False
    standalone_name: str | None = None
    support_servers: tuple[str, ...] | None = None
    packages: tuple[str, ...] | None = None

    def __hash__(self) -> int:
        return hash(self.name)

    def __eq__(self, another):
        return (
            False
            if another is None
            else (
                self.name == another.name
                if isinstance(another, ServiceDescriptionBase)
                else self.name == another
            )
        )


@dataclass
class SubscriberInformation:
    type: int | None = None
    name: str | None = None
    available: bool = True
    enabled: bool = True
    service_information: ServiceDescriptionBase | None = None


@dataclass
class SubscribtionResult:
    result: Any | None = None
    type: int = 0
    checker: bool = False
